@@ -1,3 +1,4 @@
+# ruff: noqa: F401
 """
 .. include:: ../README.md
 
@@ -22,6 +23,7 @@ from sqlglot.expressions import (
     Expression as Expression,
     alias_ as alias,
     and_ as and_,
+    case as case,
     cast as cast,
     column as column,
     condition as condition,
@@ -82,18 +84,15 @@ def parse(
     Returns:
         The resulting syntax tree collection.
     """
-    dialect = Dialect.get_or_raise(read or dialect)()
-    return dialect.parse(sql, **opts)
+    return Dialect.get_or_raise(read or dialect).parse(sql, **opts)
 
 
 @t.overload
-def parse_one(sql: str, *, into: t.Type[E], **opts) -> E:
-    ...
+def parse_one(sql: str, *, into: t.Type[E], **opts) -> E: ...
 
 
 @t.overload
-def parse_one(sql: str, **opts) -> Expression:
-    ...
+def parse_one(sql: str, **opts) -> Expression: ...
 
 
 def parse_one(
@@ -117,7 +116,7 @@ def parse_one(
         The syntax tree for the first parsed statement.
     """
 
-    dialect = Dialect.get_or_raise(read or dialect)()
+    dialect = Dialect.get_or_raise(read or dialect)
 
     if into:
         result = dialect.parse_into(into, sql, **opts)
@@ -157,7 +156,8 @@ def transpile(
         The list of transpiled SQL statements.
     """
     write = (read if write is None else write) if identity else write
+    write = Dialect.get_or_raise(write)
     return [
-        Dialect.get_or_raise(write)().generate(expression, **opts)
+        write.generate(expression, copy=False, **opts) if expression else ""
         for expression in parse(sql, read, error_level=error_level)
     ]
