@@ -148,7 +148,7 @@ def atanh(col: ColumnOrName) -> Column:
 
 
 def cbrt(col: ColumnOrName) -> Column:
-    return Column.invoke_anonymous_function(col, "CBRT")
+    return Column.invoke_expression_over_column(col, expression.Cbrt)
 
 
 def ceil(col: ColumnOrName) -> Column:
@@ -210,7 +210,7 @@ def sec(col: ColumnOrName) -> Column:
 
 
 def signum(col: ColumnOrName) -> Column:
-    return Column.invoke_anonymous_function(col, "SIGNUM")
+    return Column.invoke_expression_over_column(col, expression.Sign)
 
 
 def sin(col: ColumnOrName) -> Column:
@@ -368,7 +368,10 @@ def covar_samp(col1: ColumnOrName, col2: ColumnOrName) -> Column:
 
 
 def first(col: ColumnOrName, ignorenulls: t.Optional[bool] = None) -> Column:
-    return Column.invoke_expression_over_column(col, expression.First, ignore_nulls=ignorenulls)
+    this = Column.invoke_expression_over_column(col, expression.First)
+    if ignorenulls:
+        return Column.invoke_expression_over_column(this, expression.IgnoreNulls)
+    return this
 
 
 def grouping_id(*cols: ColumnOrName) -> Column:
@@ -392,7 +395,10 @@ def isnull(col: ColumnOrName) -> Column:
 
 
 def last(col: ColumnOrName, ignorenulls: t.Optional[bool] = None) -> Column:
-    return Column.invoke_expression_over_column(col, expression.Last, ignore_nulls=ignorenulls)
+    this = Column.invoke_expression_over_column(col, expression.Last)
+    if ignorenulls:
+        return Column.invoke_expression_over_column(this, expression.IgnoreNulls)
+    return this
 
 
 def monotonically_increasing_id() -> Column:
@@ -485,31 +491,28 @@ def factorial(col: ColumnOrName) -> Column:
 def lag(
     col: ColumnOrName, offset: t.Optional[int] = 1, default: t.Optional[ColumnOrLiteral] = None
 ) -> Column:
-    if default is not None:
-        return Column.invoke_anonymous_function(col, "LAG", offset, default)
-    if offset != 1:
-        return Column.invoke_anonymous_function(col, "LAG", offset)
-    return Column.invoke_anonymous_function(col, "LAG")
+    return Column.invoke_expression_over_column(
+        col, expression.Lag, offset=None if offset == 1 else offset, default=default
+    )
 
 
 def lead(
     col: ColumnOrName, offset: t.Optional[int] = 1, default: t.Optional[t.Any] = None
 ) -> Column:
-    if default is not None:
-        return Column.invoke_anonymous_function(col, "LEAD", offset, default)
-    if offset != 1:
-        return Column.invoke_anonymous_function(col, "LEAD", offset)
-    return Column.invoke_anonymous_function(col, "LEAD")
+    return Column.invoke_expression_over_column(
+        col, expression.Lead, offset=None if offset == 1 else offset, default=default
+    )
 
 
 def nth_value(
     col: ColumnOrName, offset: t.Optional[int] = 1, ignoreNulls: t.Optional[bool] = None
 ) -> Column:
+    this = Column.invoke_expression_over_column(
+        col, expression.NthValue, offset=None if offset == 1 else offset
+    )
     if ignoreNulls is not None:
-        raise NotImplementedError("There is currently not support for `ignoreNulls` parameter")
-    if offset != 1:
-        return Column.invoke_anonymous_function(col, "NTH_VALUE", offset)
-    return Column.invoke_anonymous_function(col, "NTH_VALUE")
+        return Column.invoke_expression_over_column(this, expression.IgnoreNulls)
+    return this
 
 
 def ntile(n: int) -> Column:
@@ -574,13 +577,13 @@ def make_date(year: ColumnOrName, month: ColumnOrName, day: ColumnOrName) -> Col
 
 def date_add(col: ColumnOrName, days: t.Union[ColumnOrName, int]) -> Column:
     return Column.invoke_expression_over_column(
-        col, expression.DateAdd, expression=days, unit=expression.Var(this="day")
+        col, expression.DateAdd, expression=days, unit=expression.Var(this="DAY")
     )
 
 
 def date_sub(col: ColumnOrName, days: t.Union[ColumnOrName, int]) -> Column:
     return Column.invoke_expression_over_column(
-        col, expression.DateSub, expression=days, unit=expression.Var(this="day")
+        col, expression.DateSub, expression=days, unit=expression.Var(this="DAY")
     )
 
 
@@ -589,7 +592,7 @@ def date_diff(end: ColumnOrName, start: ColumnOrName) -> Column:
 
 
 def add_months(start: ColumnOrName, months: t.Union[ColumnOrName, int]) -> Column:
-    return Column.invoke_anonymous_function(start, "ADD_MONTHS", months)
+    return Column.invoke_expression_over_column(start, expression.AddMonths, expression=months)
 
 
 def months_between(
@@ -635,7 +638,7 @@ def next_day(col: ColumnOrName, dayOfWeek: str) -> Column:
 
 
 def last_day(col: ColumnOrName) -> Column:
-    return Column.invoke_anonymous_function(col, "LAST_DAY")
+    return Column.invoke_expression_over_column(col, expression.LastDay)
 
 
 def from_unixtime(col: ColumnOrName, format: t.Optional[str] = None) -> Column:
@@ -661,7 +664,7 @@ def from_utc_timestamp(timestamp: ColumnOrName, tz: ColumnOrName) -> Column:
 
 def to_utc_timestamp(timestamp: ColumnOrName, tz: ColumnOrName) -> Column:
     tz_column = tz if isinstance(tz, Column) else lit(tz)
-    return Column.invoke_anonymous_function(timestamp, "TO_UTC_TIMESTAMP", tz_column)
+    return Column.invoke_expression_over_column(timestamp, expression.FromTimeZone, zone=tz_column)
 
 
 def timestamp_seconds(col: ColumnOrName) -> Column:
@@ -1027,7 +1030,7 @@ def posexplode_outer(col: ColumnOrName) -> Column:
 
 
 def get_json_object(col: ColumnOrName, path: str) -> Column:
-    return Column.invoke_expression_over_column(col, expression.JSONExtract, path=lit(path))
+    return Column.invoke_expression_over_column(col, expression.JSONExtract, expression=lit(path))
 
 
 def json_tuple(col: ColumnOrName, *fields: str) -> Column:
