@@ -17,11 +17,26 @@ class DataFrameReader:
 
     def table(self, tableName: str) -> DataFrame:
         from sqlglot.dataframe.sql.dataframe import DataFrame
+        from sqlglot.dataframe.sql.session import SparkSession
+
+        sqlglot.schema.add_table(tableName, dialect=SparkSession().dialect)
 
-        sqlglot.schema.add_table(tableName)
         return DataFrame(
             self.spark,
-            exp.Select().from_(tableName).select(*sqlglot.schema.column_names(tableName)),
+            exp.Select()
+            .from_(
+                exp.to_table(tableName, dialect=SparkSession().dialect).transform(
+                    SparkSession().dialect.normalize_identifier
+                )
+            )
+            .select(
+                *(
+                    column
+                    for column in sqlglot.schema.column_names(
+                        tableName, dialect=SparkSession().dialect
+                    )
+                )
+            ),
         )
 
 
@@ -57,6 +72,8 @@ class DataFrameWriter:
         return self.copy(by_name=True)
 
     def insertInto(self, tableName: str, overwrite: t.Optional[bool] = None) -> DataFrameWriter:
+        from sqlglot.dataframe.sql.session import SparkSession
+
         output_expression_container = exp.Insert(
             **{
                 "this": exp.to_table(tableName),
@@ -65,7 +82,9 @@ class DataFrameWriter:
         )
         df = self._df.copy(output_expression_container=output_expression_container)
         if self._by_name:
-            columns = sqlglot.schema.column_names(tableName, only_visible=True)
+            columns = sqlglot.schema.column_names(
+                tableName, only_visible=True, dialect=SparkSession().dialect
+            )
             df = df._convert_leaf_to_cte().select(*columns)
 
         return self.copy(_df=df)
