@@ -70,21 +70,21 @@ class SparkSession:
             column_mapping = {f"_{i}": None for i in range(1, len(data[0]) + 1)}
 
         data_expressions = [
-            exp.Tuple(
-                expressions=list(
-                    map(
-                        lambda x: F.lit(x).expression,
-                        row if not isinstance(row, dict) else row.values(),
-                    )
+            exp.tuple_(
+                *map(
+                    lambda x: F.lit(x).expression,
+                    row if not isinstance(row, dict) else row.values(),
                 )
             )
             for row in data
         ]
 
         sel_columns = [
-            F.col(name).cast(data_type).alias(name).expression
-            if data_type is not None
-            else F.col(name).expression
+            (
+                F.col(name).cast(data_type).alias(name).expression
+                if data_type is not None
+                else F.col(name).expression
+            )
             for name, data_type in column_mapping.items()
         ]
 
