@@ -27,6 +27,6 @@ def get_tables_from_expression_with_join(expression: exp.Select) -> t.List[exp.T
     if not expression.args.get("joins"):
         return []
 
-    left_table = expression.args["from"].args["expressions"][0]
+    left_table = expression.args["from"].this
     other_tables = [join.this for join in expression.args["joins"]]
     return [left_table] + other_tables
