@@ -6,14 +6,17 @@ from sqlglot import exp, generator, parser, tokens, transforms
 from sqlglot.dialects.dialect import (
     Dialect,
     arg_max_or_min_no_count,
+    date_delta_sql,
     inline_array_sql,
+    json_extract_segments,
+    json_path_key_only_name,
     no_pivot_sql,
+    build_json_extract_path,
     rename_func,
     var_map_sql,
 )
 from sqlglot.errors import ParseError
-from sqlglot.helper import seq_get
-from sqlglot.parser import parse_var_map
+from sqlglot.helper import is_int, seq_get
 from sqlglot.tokens import Token, TokenType
 
 
@@ -22,16 +25,25 @@ def _lower_func(sql: str) -> str:
     return sql[:index].lower() + sql[index:]
 
 
-def _quantile_sql(self, e):
-    quantile = e.args["quantile"]
-    args = f"({self.sql(e, 'this')})"
+def _quantile_sql(self: ClickHouse.Generator, expression: exp.Quantile) -> str:
+    quantile = expression.args["quantile"]
+    args = f"({self.sql(expression, 'this')})"
+
     if isinstance(quantile, exp.Array):
         func = self.func("quantiles", *quantile)
     else:
         func = self.func("quantile", quantile)
+
     return func + args
 
 
+def _build_count_if(args: t.List) -> exp.CountIf | exp.CombinedAggFunc:
+    if len(args) == 1:
+        return exp.CountIf(this=seq_get(args, 0))
+
+    return exp.CombinedAggFunc(this="countIf", expressions=args, parts=("count", "If"))
+
+
 class ClickHouse(Dialect):
     NORMALIZE_FUNCTIONS: bool | str = False
     NULL_ORDERING = "nulls_are_last"
@@ -53,9 +65,9 @@ class ClickHouse(Dialect):
         KEYWORDS = {
             **tokens.Tokenizer.KEYWORDS,
             "ATTACH": TokenType.COMMAND,
+            "DATE32": TokenType.DATE32,
             "DATETIME64": TokenType.DATETIME64,
             "DICTIONARY": TokenType.DICTIONARY,
-            "ENUM": TokenType.ENUM,
             "ENUM8": TokenType.ENUM8,
             "ENUM16": TokenType.ENUM16,
             "FINAL": TokenType.FINAL,
@@ -75,6 +87,12 @@ class ClickHouse(Dialect):
             "UINT32": TokenType.UINT,
             "UINT64": TokenType.UBIGINT,
             "UINT8": TokenType.UTINYINT,
+            "IPV4": TokenType.IPV4,
+            "IPV6": TokenType.IPV6,
+            "AGGREGATEFUNCTION": TokenType.AGGREGATEFUNCTION,
+            "SIMPLEAGGREGATEFUNCTION": TokenType.SIMPLEAGGREGATEFUNCTION,
+            "SYSTEM": TokenType.COMMAND,
+            "PREWHERE": TokenType.PREWHERE,
         }
 
         SINGLE_TOKENS = {
@@ -91,6 +109,8 @@ class ClickHouse(Dialect):
         FUNCTIONS = {
             **parser.Parser.FUNCTIONS,
             "ANY": exp.AnyValue.from_arg_list,
+            "ARRAYSUM": exp.ArraySum.from_arg_list,
+            "COUNTIF": _build_count_if,
             "DATE_ADD": lambda args: exp.DateAdd(
                 this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
             ),
@@ -103,13 +123,149 @@ class ClickHouse(Dialect):
             "DATEDIFF": lambda args: exp.DateDiff(
                 this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
             ),
-            "MAP": parse_var_map,
+            "JSONEXTRACTSTRING": build_json_extract_path(
+                exp.JSONExtractScalar, zero_based_indexing=False
+            ),
+            "MAP": parser.build_var_map,
             "MATCH": exp.RegexpLike.from_arg_list,
             "RANDCANONICAL": exp.Rand.from_arg_list,
+            "TUPLE": exp.Struct.from_arg_list,
             "UNIQ": exp.ApproxDistinct.from_arg_list,
             "XOR": lambda args: exp.Xor(expressions=args),
         }
 
+        AGG_FUNCTIONS = {
+            "count",
+            "min",
+            "max",
+            "sum",
+            "avg",
+            "any",
+            "stddevPop",
+            "stddevSamp",
+            "varPop",
+            "varSamp",
+            "corr",
+            "covarPop",
+            "covarSamp",
+            "entropy",
+            "exponentialMovingAverage",
+            "intervalLengthSum",
+            "kolmogorovSmirnovTest",
+            "mannWhitneyUTest",
+            "median",
+            "rankCorr",
+            "sumKahan",
+            "studentTTest",
+            "welchTTest",
+            "anyHeavy",
+            "anyLast",
+            "boundingRatio",
+            "first_value",
+            "last_value",
+            "argMin",
+            "argMax",
+            "avgWeighted",
+            "topK",
+            "topKWeighted",
+            "deltaSum",
+            "deltaSumTimestamp",
+            "groupArray",
+            "groupArrayLast",
+            "groupUniqArray",
+            "groupArrayInsertAt",
+            "groupArrayMovingAvg",
+            "groupArrayMovingSum",
+            "groupArraySample",
+            "groupBitAnd",
+            "groupBitOr",
+            "groupBitXor",
+            "groupBitmap",
+            "groupBitmapAnd",
+            "groupBitmapOr",
+            "groupBitmapXor",
+            "sumWithOverflow",
+            "sumMap",
+            "minMap",
+            "maxMap",
+            "skewSamp",
+            "skewPop",
+            "kurtSamp",
+            "kurtPop",
+            "uniq",
+            "uniqExact",
+            "uniqCombined",
+            "uniqCombined64",
+            "uniqHLL12",
+            "uniqTheta",
+            "quantile",
+            "quantiles",
+            "quantileExact",
+            "quantilesExact",
+            "quantileExactLow",
+            "quantilesExactLow",
+            "quantileExactHigh",
+            "quantilesExactHigh",
+            "quantileExactWeighted",
+            "quantilesExactWeighted",
+            "quantileTiming",
+            "quantilesTiming",
+            "quantileTimingWeighted",
+            "quantilesTimingWeighted",
+            "quantileDeterministic",
+            "quantilesDeterministic",
+            "quantileTDigest",
+            "quantilesTDigest",
+            "quantileTDigestWeighted",
+            "quantilesTDigestWeighted",
+            "quantileBFloat16",
+            "quantilesBFloat16",
+            "quantileBFloat16Weighted",
+            "quantilesBFloat16Weighted",
+            "simpleLinearRegression",
+            "stochasticLinearRegression",
+            "stochasticLogisticRegression",
+            "categoricalInformationValue",
+            "contingency",
+            "cramersV",
+            "cramersVBiasCorrected",
+            "theilsU",
+            "maxIntersections",
+            "maxIntersectionsPosition",
+            "meanZTest",
+            "quantileInterpolatedWeighted",
+            "quantilesInterpolatedWeighted",
+            "quantileGK",
+            "quantilesGK",
+            "sparkBar",
+            "sumCount",
+            "largestTriangleThreeBuckets",
+        }
+
+        AGG_FUNCTIONS_SUFFIXES = [
+            "If",
+            "Array",
+            "ArrayIf",
+            "Map",
+            "SimpleState",
+            "State",
+            "Merge",
+            "MergeState",
+            "ForEach",
+            "Distinct",
+            "OrDefault",
+            "OrNull",
+            "Resample",
+            "ArgMin",
+            "ArgMax",
+        ]
+
+        AGG_FUNC_MAPPING = (
+            lambda functions, suffixes: {
+                f"{f}{sfx}": (f, sfx) for sfx in (suffixes + [""]) for f in functions
+            }
+        )(AGG_FUNCTIONS, AGG_FUNCTIONS_SUFFIXES)
+
         FUNCTIONS_WITH_ALIASED_ARGS = {*parser.Parser.FUNCTIONS_WITH_ALIASED_ARGS, "TUPLE"}
 
         FUNCTION_PARSERS = {
@@ -205,9 +361,14 @@ class ClickHouse(Dialect):
             joins: bool = False,
             alias_tokens: t.Optional[t.Collection[TokenType]] = None,
             parse_bracket: bool = False,
+            is_db_reference: bool = False,
         ) -> t.Optional[exp.Expression]:
             this = super()._parse_table(
-                schema=schema, joins=joins, alias_tokens=alias_tokens, parse_bracket=parse_bracket
+                schema=schema,
+                joins=joins,
+                alias_tokens=alias_tokens,
+                parse_bracket=parse_bracket,
+                is_db_reference=is_db_reference,
             )
 
             if self._match(TokenType.FINAL):
@@ -230,7 +391,7 @@ class ClickHouse(Dialect):
 
                 return self.expression(
                     exp.CTE,
-                    this=self._parse_field(),
+                    this=self._parse_conjunction(),
                     alias=self._parse_table_alias(),
                     scalar=True,
                 )
@@ -256,9 +417,9 @@ class ClickHouse(Dialect):
             self, skip_join_token: bool = False, parse_bracket: bool = False
         ) -> t.Optional[exp.Join]:
             join = super()._parse_join(skip_join_token=skip_join_token, parse_bracket=True)
-
             if join:
                 join.set("global", join.args.pop("method", None))
+
             return join
 
         def _parse_function(
@@ -272,9 +433,18 @@ class ClickHouse(Dialect):
             )
 
             if isinstance(func, exp.Anonymous):
+                parts = self.AGG_FUNC_MAPPING.get(func.this)
                 params = self._parse_func_params(func)
 
                 if params:
+                    if parts and parts[1]:
+                        return self.expression(
+                            exp.CombinedParameterizedAgg,
+                            this=func.this,
+                            expressions=func.expressions,
+                            params=params,
+                            parts=parts,
+                        )
                     return self.expression(
                         exp.ParameterizedAgg,
                         this=func.this,
@@ -282,6 +452,20 @@ class ClickHouse(Dialect):
                         params=params,
                     )
 
+                if parts:
+                    if parts[1]:
+                        return self.expression(
+                            exp.CombinedAggFunc,
+                            this=func.this,
+                            expressions=func.expressions,
+                            parts=parts,
+                        )
+                    return self.expression(
+                        exp.AnonymousAggFunc,
+                        this=func.this,
+                        expressions=func.expressions,
+                    )
+
             return func
 
         def _parse_func_params(
@@ -329,6 +513,10 @@ class ClickHouse(Dialect):
         STRUCT_DELIMITER = ("(", ")")
         NVL2_SUPPORTED = False
         TABLESAMPLE_REQUIRES_PARENS = False
+        TABLESAMPLE_SIZE_IS_ROWS = False
+        TABLESAMPLE_KEYWORDS = "SAMPLE"
+        LAST_DAY_SUPPORTS_DATE_PART = False
+        CAN_IMPLEMENT_ARRAY_ANY = True
 
         STRING_TYPE_MAPPING = {
             exp.DataType.Type.CHAR: "String",
@@ -343,11 +531,18 @@ class ClickHouse(Dialect):
             exp.DataType.Type.VARCHAR: "String",
         }
 
+        SUPPORTED_JSON_PATH_PARTS = {
+            exp.JSONPathKey,
+            exp.JSONPathRoot,
+            exp.JSONPathSubscript,
+        }
+
         TYPE_MAPPING = {
             **generator.Generator.TYPE_MAPPING,
             **STRING_TYPE_MAPPING,
             exp.DataType.Type.ARRAY: "Array",
             exp.DataType.Type.BIGINT: "Int64",
+            exp.DataType.Type.DATE32: "Date32",
             exp.DataType.Type.DATETIME64: "DateTime64",
             exp.DataType.Type.DOUBLE: "Float64",
             exp.DataType.Type.ENUM: "Enum",
@@ -372,36 +567,46 @@ class ClickHouse(Dialect):
             exp.DataType.Type.UINT256: "UInt256",
             exp.DataType.Type.USMALLINT: "UInt16",
             exp.DataType.Type.UTINYINT: "UInt8",
+            exp.DataType.Type.IPV4: "IPv4",
+            exp.DataType.Type.IPV6: "IPv6",
+            exp.DataType.Type.AGGREGATEFUNCTION: "AggregateFunction",
+            exp.DataType.Type.SIMPLEAGGREGATEFUNCTION: "SimpleAggregateFunction",
         }
 
         TRANSFORMS = {
             **generator.Generator.TRANSFORMS,
-            exp.Select: transforms.preprocess([transforms.eliminate_qualify]),
             exp.AnyValue: rename_func("any"),
             exp.ApproxDistinct: rename_func("uniq"),
+            exp.ArrayFilter: lambda self, e: self.func("arrayFilter", e.expression, e.this),
+            exp.ArraySize: rename_func("LENGTH"),
+            exp.ArraySum: rename_func("arraySum"),
             exp.ArgMax: arg_max_or_min_no_count("argMax"),
             exp.ArgMin: arg_max_or_min_no_count("argMin"),
             exp.Array: inline_array_sql,
             exp.CastToStrType: rename_func("CAST"),
+            exp.CountIf: rename_func("countIf"),
             exp.CurrentDate: lambda self, e: self.func("CURRENT_DATE"),
-            exp.DateAdd: lambda self, e: self.func(
-                "DATE_ADD", exp.Literal.string(e.text("unit") or "day"), e.expression, e.this
-            ),
-            exp.DateDiff: lambda self, e: self.func(
-                "DATE_DIFF", exp.Literal.string(e.text("unit") or "day"), e.expression, e.this
-            ),
+            exp.DateAdd: date_delta_sql("DATE_ADD"),
+            exp.DateDiff: date_delta_sql("DATE_DIFF"),
             exp.Explode: rename_func("arrayJoin"),
             exp.Final: lambda self, e: f"{self.sql(e, 'this')} FINAL",
             exp.IsNan: rename_func("isNaN"),
+            exp.JSONExtract: json_extract_segments("JSONExtractString", quoted_index=False),
+            exp.JSONExtractScalar: json_extract_segments("JSONExtractString", quoted_index=False),
+            exp.JSONPathKey: json_path_key_only_name,
+            exp.JSONPathRoot: lambda *_: "",
             exp.Map: lambda self, e: _lower_func(var_map_sql(self, e)),
             exp.Nullif: rename_func("nullIf"),
             exp.PartitionedByProperty: lambda self, e: f"PARTITION BY {self.sql(e, 'this')}",
             exp.Pivot: no_pivot_sql,
             exp.Quantile: _quantile_sql,
-            exp.RegexpLike: lambda self, e: f"match({self.format_args(e.this, e.expression)})",
+            exp.RegexpLike: lambda self, e: self.func("match", e.this, e.expression),
             exp.Rand: rename_func("randCanonical"),
+            exp.Select: transforms.preprocess([transforms.eliminate_qualify]),
             exp.StartsWith: rename_func("startsWith"),
-            exp.StrPosition: lambda self, e: f"position({self.format_args(e.this, e.args.get('substr'), e.args.get('position'))})",
+            exp.StrPosition: lambda self, e: self.func(
+                "position", e.this, e.args.get("substr"), e.args.get("position")
+            ),
             exp.VarMap: lambda self, e: _lower_func(var_map_sql(self, e)),
             exp.Xor: lambda self, e: self.func("xor", e.this, e.expression, *e.expressions),
         }
@@ -430,6 +635,13 @@ class ClickHouse(Dialect):
             "NAMED COLLECTION",
         }
 
+        def _jsonpathsubscript_sql(self, expression: exp.JSONPathSubscript) -> str:
+            this = self.json_path_part(expression.this)
+            return str(int(this) + 1) if is_int(this) else this
+
+        def likeproperty_sql(self, expression: exp.LikeProperty) -> str:
+            return f"AS {self.sql(expression, 'this')}"
+
         def _any_to_has(
             self,
             expression: exp.EQ | exp.NEQ,
@@ -444,6 +656,7 @@ class ClickHouse(Dialect):
                 this = expression.left
             else:
                 return default(expression)
+
             return prefix + self.func("has", arr.this.unnest(), this)
 
         def eq_sql(self, expression: exp.EQ) -> str:
@@ -455,7 +668,7 @@ class ClickHouse(Dialect):
         def regexpilike_sql(self, expression: exp.RegexpILike) -> str:
             # Manually add a flag to make the search case-insensitive
             regex = self.func("CONCAT", "'(?i)'", expression.expression)
-            return f"match({self.format_args(expression.this, regex)})"
+            return self.func("match", expression.this, regex)
 
         def datatype_sql(self, expression: exp.DataType) -> str:
             # String is the standard ClickHouse type, every other variant is just an alias.
@@ -477,18 +690,31 @@ class ClickHouse(Dialect):
 
         def after_limit_modifiers(self, expression: exp.Expression) -> t.List[str]:
             return super().after_limit_modifiers(expression) + [
-                self.seg("SETTINGS ") + self.expressions(expression, key="settings", flat=True)
-                if expression.args.get("settings")
-                else "",
-                self.seg("FORMAT ") + self.sql(expression, "format")
-                if expression.args.get("format")
-                else "",
+                (
+                    self.seg("SETTINGS ") + self.expressions(expression, key="settings", flat=True)
+                    if expression.args.get("settings")
+                    else ""
+                ),
+                (
+                    self.seg("FORMAT ") + self.sql(expression, "format")
+                    if expression.args.get("format")
+                    else ""
+                ),
             ]
 
-        def parameterizedagg_sql(self, expression: exp.Anonymous) -> str:
+        def parameterizedagg_sql(self, expression: exp.ParameterizedAgg) -> str:
             params = self.expressions(expression, key="params", flat=True)
             return self.func(expression.name, *expression.expressions) + f"({params})"
 
+        def anonymousaggfunc_sql(self, expression: exp.AnonymousAggFunc) -> str:
+            return self.func(expression.name, *expression.expressions)
+
+        def combinedaggfunc_sql(self, expression: exp.CombinedAggFunc) -> str:
+            return self.anonymousaggfunc_sql(expression)
+
+        def combinedparameterizedagg_sql(self, expression: exp.CombinedParameterizedAgg) -> str:
+            return self.parameterizedagg_sql(expression)
+
         def placeholder_sql(self, expression: exp.Placeholder) -> str:
             return f"{{{expression.name}: {self.sql(expression, 'kind')}}}"
 
@@ -496,8 +722,9 @@ class ClickHouse(Dialect):
             return f"ON CLUSTER {self.sql(expression, 'this')}"
 
         def createable_sql(self, expression: exp.Create, locations: t.DefaultDict) -> str:
-            kind = self.sql(expression, "kind").upper()
-            if kind in self.ON_CLUSTER_TARGETS and locations.get(exp.Properties.Location.POST_NAME):
+            if expression.kind in self.ON_CLUSTER_TARGETS and locations.get(
+                exp.Properties.Location.POST_NAME
+            ):
                 this_name = self.sql(expression.this, "this")
                 this_properties = " ".join(
                     [self.sql(prop) for prop in locations[exp.Properties.Location.POST_NAME]]
@@ -506,3 +733,7 @@ class ClickHouse(Dialect):
                 return f"{this_name}{self.sep()}{this_properties}{self.sep()}{this_schema}"
 
             return super().createable_sql(expression, locations)
+
+        def prewhere_sql(self, expression: exp.PreWhere) -> str:
+            this = self.indent(self.sql(expression, "this"))
+            return f"{self.seg('PREWHERE')}{self.sep()}{this}"
