@@ -6,12 +6,11 @@ from sqlglot import exp, generator, parser, tokens, transforms
 from sqlglot.dialects.dialect import (
     Dialect,
     NormalizationStrategy,
-    arrow_json_extract_scalar_sql,
+    arrow_json_extract_sql,
     date_add_interval_sql,
     datestrtodate_sql,
-    format_time_lambda,
+    build_formatted_time,
     isnull_to_is_null,
-    json_keyvalue_comma_sql,
     locate_to_strposition,
     max_or_greatest,
     min_or_least,
@@ -20,7 +19,8 @@ from sqlglot.dialects.dialect import (
     no_pivot_sql,
     no_tablesample_sql,
     no_trycast_sql,
-    parse_date_delta_with_interval,
+    build_date_delta,
+    build_date_delta_with_interval,
     rename_func,
     strposition_to_locate_sql,
 )
@@ -37,28 +37,26 @@ def _show_parser(*args: t.Any, **kwargs: t.Any) -> t.Callable[[MySQL.Parser], ex
 
 def _date_trunc_sql(self: MySQL.Generator, expression: exp.DateTrunc) -> str:
     expr = self.sql(expression, "this")
-    unit = expression.text("unit")
+    unit = expression.text("unit").upper()
 
-    if unit == "day":
-        return f"DATE({expr})"
-
-    if unit == "week":
+    if unit == "WEEK":
         concat = f"CONCAT(YEAR({expr}), ' ', WEEK({expr}, 1), ' 1')"
         date_format = "%Y %u %w"
-    elif unit == "month":
+    elif unit == "MONTH":
         concat = f"CONCAT(YEAR({expr}), ' ', MONTH({expr}), ' 1')"
         date_format = "%Y %c %e"
-    elif unit == "quarter":
+    elif unit == "QUARTER":
         concat = f"CONCAT(YEAR({expr}), ' ', QUARTER({expr}) * 3 - 2, ' 1')"
         date_format = "%Y %c %e"
-    elif unit == "year":
+    elif unit == "YEAR":
         concat = f"CONCAT(YEAR({expr}), ' 1 1')"
         date_format = "%Y %c %e"
     else:
-        self.unsupported(f"Unexpected interval unit: {unit}")
-        return f"DATE({expr})"
+        if unit != "DAY":
+            self.unsupported(f"Unexpected interval unit: {unit}")
+        return self.func("DATE", expr)
 
-    return f"STR_TO_DATE({concat}, '{date_format}')"
+    return self.func("STR_TO_DATE", concat, f"'{date_format}'")
 
 
 # All specifiers for time parts (as opposed to date parts)
@@ -93,8 +91,7 @@ def _str_to_date(args: t.List) -> exp.StrToDate | exp.StrToTime:
 def _str_to_date_sql(
     self: MySQL.Generator, expression: exp.StrToDate | exp.StrToTime | exp.TsOrDsToDate
 ) -> str:
-    date_format = self.format_time(expression)
-    return f"STR_TO_DATE({self.sql(expression.this)}, {date_format})"
+    return self.func("STR_TO_DATE", expression.this, self.format_time(expression))
 
 
 def _trim_sql(self: MySQL.Generator, expression: exp.Trim) -> str:
@@ -127,9 +124,7 @@ def _date_add_sql(
 
 def _ts_or_ds_to_date_sql(self: MySQL.Generator, expression: exp.TsOrDsToDate) -> str:
     time_format = expression.args.get("format")
-    if time_format:
-        return _str_to_date_sql(self, expression)
-    return f"DATE({self.sql(expression, 'this')})"
+    return _str_to_date_sql(self, expression) if time_format else self.func("DATE", expression.this)
 
 
 def _remove_ts_or_ds_to_date(
@@ -190,7 +185,6 @@ class MySQL(Dialect):
         KEYWORDS = {
             **tokens.Tokenizer.KEYWORDS,
             "CHARSET": TokenType.CHARACTER_SET,
-            "ENUM": TokenType.ENUM,
             "FORCE": TokenType.FORCE,
             "IGNORE": TokenType.IGNORE,
             "LOCK TABLES": TokenType.COMMAND,
@@ -289,17 +283,24 @@ class MySQL(Dialect):
         FUNCTIONS = {
             **parser.Parser.FUNCTIONS,
             "DATE": lambda args: exp.TsOrDsToDate(this=seq_get(args, 0)),
-            "DATE_ADD": parse_date_delta_with_interval(exp.DateAdd),
-            "DATE_FORMAT": format_time_lambda(exp.TimeToStr, "mysql"),
-            "DATE_SUB": parse_date_delta_with_interval(exp.DateSub),
+            "DATE_ADD": build_date_delta_with_interval(exp.DateAdd),
+            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "mysql"),
+            "DATE_SUB": build_date_delta_with_interval(exp.DateSub),
+            "DAY": lambda args: exp.Day(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
+            "DAYOFMONTH": lambda args: exp.DayOfMonth(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
+            "DAYOFWEEK": lambda args: exp.DayOfWeek(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
+            "DAYOFYEAR": lambda args: exp.DayOfYear(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
             "INSTR": lambda args: exp.StrPosition(substr=seq_get(args, 1), this=seq_get(args, 0)),
             "ISNULL": isnull_to_is_null,
             "LOCATE": locate_to_strposition,
+            "MAKETIME": exp.TimeFromParts.from_arg_list,
+            "MONTH": lambda args: exp.Month(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
             "MONTHNAME": lambda args: exp.TimeToStr(
                 this=exp.TsOrDsToDate(this=seq_get(args, 0)),
                 format=exp.Literal.string("%B"),
             ),
             "STR_TO_DATE": _str_to_date,
+            "TIMESTAMPDIFF": build_date_delta(exp.TimestampDiff),
             "TO_DAYS": lambda args: exp.paren(
                 exp.DateDiff(
                     this=exp.TsOrDsToDate(this=seq_get(args, 0)),
@@ -308,11 +309,6 @@ class MySQL(Dialect):
                 )
                 + 1
             ),
-            "DAY": lambda args: exp.Day(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
-            "DAYOFMONTH": lambda args: exp.DayOfMonth(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
-            "DAYOFWEEK": lambda args: exp.DayOfWeek(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
-            "DAYOFYEAR": lambda args: exp.DayOfYear(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
-            "MONTH": lambda args: exp.Month(this=exp.TsOrDsToDate(this=seq_get(args, 0))),
             "WEEK": lambda args: exp.Week(
                 this=exp.TsOrDsToDate(this=seq_get(args, 0)), mode=seq_get(args, 1)
             ),
@@ -356,6 +352,7 @@ class MySQL(Dialect):
             "CREATE TRIGGER": _show_parser("CREATE TRIGGER", target=True),
             "CREATE VIEW": _show_parser("CREATE VIEW", target=True),
             "DATABASES": _show_parser("DATABASES"),
+            "SCHEMAS": _show_parser("DATABASES"),
             "ENGINE": _show_parser("ENGINE", target=True),
             "STORAGE ENGINES": _show_parser("ENGINES"),
             "ENGINES": _show_parser("ENGINES"),
@@ -393,6 +390,11 @@ class MySQL(Dialect):
             "WARNINGS": _show_parser("WARNINGS"),
         }
 
+        PROPERTY_PARSERS = {
+            **parser.Parser.PROPERTY_PARSERS,
+            "LOCK": lambda self: self._parse_property_assignment(exp.LockProperty),
+        }
+
         SET_PARSERS = {
             **parser.Parser.SET_PARSERS,
             "PERSIST": lambda self: self._parse_set_item_assignment("PERSIST"),
@@ -418,16 +420,11 @@ class MySQL(Dialect):
             "SPATIAL",
         }
 
-        PROFILE_TYPES = {
-            "ALL",
-            "BLOCK IO",
-            "CONTEXT SWITCHES",
-            "CPU",
-            "IPC",
-            "MEMORY",
-            "PAGE FAULTS",
-            "SOURCE",
-            "SWAPS",
+        PROFILE_TYPES: parser.OPTIONS_TYPE = {
+            **dict.fromkeys(("ALL", "CPU", "IPC", "MEMORY", "SOURCE", "SWAPS"), tuple()),
+            "BLOCK": ("IO",),
+            "CONTEXT": ("SWITCHES",),
+            "PAGE": ("FAULTS",),
         }
 
         TYPE_TOKENS = {
@@ -441,6 +438,8 @@ class MySQL(Dialect):
         }
 
         LOG_DEFAULTS_TO_LN = True
+        STRING_ALIASES = True
+        VALUES_FOLLOWED_BY_PAREN = False
 
         def _parse_primary_key_part(self) -> t.Optional[exp.Expression]:
             this = self._parse_id_var()
@@ -620,13 +619,17 @@ class MySQL(Dialect):
 
     class Generator(generator.Generator):
         LOCKING_READS_SUPPORTED = True
-        NULL_ORDERING_SUPPORTED = False
+        NULL_ORDERING_SUPPORTED = None
         JOIN_HINTS = False
         TABLE_HINTS = True
         DUPLICATE_KEY_UPDATE_WITH_SET = False
         QUERY_HINT_SEP = " "
         VALUES_AS_TABLE = False
         NVL2_SUPPORTED = False
+        LAST_DAY_SUPPORTS_DATE_PART = False
+        JSON_TYPE_REQUIRED_FOR_EXTRACTION = True
+        JSON_PATH_BRACKETED_KEY_SUPPORTED = False
+        JSON_KEY_VALUE_PAIR_SEP = ","
 
         TRANSFORMS = {
             **generator.Generator.TRANSFORMS,
@@ -642,15 +645,16 @@ class MySQL(Dialect):
             exp.DayOfMonth: _remove_ts_or_ds_to_date(rename_func("DAYOFMONTH")),
             exp.DayOfWeek: _remove_ts_or_ds_to_date(rename_func("DAYOFWEEK")),
             exp.DayOfYear: _remove_ts_or_ds_to_date(rename_func("DAYOFYEAR")),
-            exp.GroupConcat: lambda self, e: f"""GROUP_CONCAT({self.sql(e, "this")} SEPARATOR {self.sql(e, "separator") or "','"})""",
+            exp.GroupConcat: lambda self,
+            e: f"""GROUP_CONCAT({self.sql(e, "this")} SEPARATOR {self.sql(e, "separator") or "','"})""",
             exp.ILike: no_ilike_sql,
-            exp.JSONExtractScalar: arrow_json_extract_scalar_sql,
-            exp.JSONKeyValue: json_keyvalue_comma_sql,
+            exp.JSONExtractScalar: arrow_json_extract_sql,
             exp.Max: max_or_greatest,
             exp.Min: min_or_least,
             exp.Month: _remove_ts_or_ds_to_date(),
             exp.NullSafeEQ: lambda self, e: self.binary(e, "<=>"),
-            exp.NullSafeNEQ: lambda self, e: self.not_sql(self.binary(e, "<=>")),
+            exp.NullSafeNEQ: lambda self, e: f"NOT {self.binary(e, '<=>')}",
+            exp.ParseJSON: lambda self, e: self.sql(e, "this"),
             exp.Pivot: no_pivot_sql,
             exp.Select: transforms.preprocess(
                 [
@@ -665,7 +669,11 @@ class MySQL(Dialect):
             exp.StrToTime: _str_to_date_sql,
             exp.Stuff: rename_func("INSERT"),
             exp.TableSample: no_tablesample_sql,
+            exp.TimeFromParts: rename_func("MAKETIME"),
             exp.TimestampAdd: date_add_interval_sql("DATE", "ADD"),
+            exp.TimestampDiff: lambda self, e: self.func(
+                "TIMESTAMPDIFF", e.text("unit"), e.expression, e.this
+            ),
             exp.TimestampSub: date_add_interval_sql("DATE", "SUB"),
             exp.TimeStrToUnix: rename_func("UNIX_TIMESTAMP"),
             exp.TimeStrToTime: lambda self, e: self.sql(exp.cast(e.this, "datetime", copy=True)),
