@@ -3,7 +3,6 @@ from __future__ import annotations
 import typing as t
 
 from sqlglot import exp, generator, parser, tokens, transforms
-from sqlglot._typing import E
 from sqlglot.dialects.dialect import (
     Dialect,
     NormalizationStrategy,
@@ -11,7 +10,7 @@ from sqlglot.dialects.dialect import (
     date_delta_sql,
     date_trunc_to_time,
     datestrtodate_sql,
-    format_time_lambda,
+    build_formatted_time,
     if_sql,
     inline_array_sql,
     max_or_greatest,
@@ -19,42 +18,24 @@ from sqlglot.dialects.dialect import (
     rename_func,
     timestamptrunc_sql,
     timestrtotime_sql,
-    ts_or_ds_to_date_sql,
     var_map_sql,
 )
 from sqlglot.expressions import Literal
-from sqlglot.helper import seq_get
+from sqlglot.helper import flatten, is_int, seq_get
 from sqlglot.tokens import TokenType
 
-
-def _check_int(s: str) -> bool:
-    if s[0] in ("-", "+"):
-        return s[1:].isdigit()
-    return s.isdigit()
+if t.TYPE_CHECKING:
+    from sqlglot._typing import E
 
 
 # from https://docs.snowflake.com/en/sql-reference/functions/to_timestamp.html
-def _parse_to_timestamp(args: t.List) -> t.Union[exp.StrToTime, exp.UnixToTime, exp.TimeStrToTime]:
+def _build_to_timestamp(args: t.List) -> t.Union[exp.StrToTime, exp.UnixToTime, exp.TimeStrToTime]:
     if len(args) == 2:
         first_arg, second_arg = args
         if second_arg.is_string:
             # case: <string_expr> [ , <format> ]
-            return format_time_lambda(exp.StrToTime, "snowflake")(args)
-
-        # case: <numeric_expr> [ , <scale> ]
-        if second_arg.name not in ["0", "3", "9"]:
-            raise ValueError(
-                f"Scale for snowflake numeric timestamp is {second_arg}, but should be 0, 3, or 9"
-            )
-
-        if second_arg.name == "0":
-            timescale = exp.UnixToTime.SECONDS
-        elif second_arg.name == "3":
-            timescale = exp.UnixToTime.MILLIS
-        elif second_arg.name == "9":
-            timescale = exp.UnixToTime.NANOS
-
-        return exp.UnixToTime(this=first_arg, scale=timescale)
+            return build_formatted_time(exp.StrToTime, "snowflake")(args)
+        return exp.UnixToTime(this=first_arg, scale=second_arg)
 
     from sqlglot.optimizer.simplify import simplify_literals
 
@@ -66,85 +47,38 @@ def _parse_to_timestamp(args: t.List) -> t.Union[exp.StrToTime, exp.UnixToTime,
         return exp.TimeStrToTime.from_arg_list(args)
 
     if first_arg.is_string:
-        if _check_int(first_arg.this):
+        if is_int(first_arg.this):
             # case: <integer>
             return exp.UnixToTime.from_arg_list(args)
 
         # case: <date_expr>
-        return format_time_lambda(exp.StrToTime, "snowflake", default=True)(args)
+        return build_formatted_time(exp.StrToTime, "snowflake", default=True)(args)
 
     # case: <numeric_expr>
     return exp.UnixToTime.from_arg_list(args)
 
 
-def _parse_object_construct(args: t.List) -> t.Union[exp.StarMap, exp.Struct]:
-    expression = parser.parse_var_map(args)
+def _build_object_construct(args: t.List) -> t.Union[exp.StarMap, exp.Struct]:
+    expression = parser.build_var_map(args)
 
     if isinstance(expression, exp.StarMap):
         return expression
 
     return exp.Struct(
         expressions=[
-            t.cast(exp.Condition, k).eq(v) for k, v in zip(expression.keys, expression.values)
+            exp.PropertyEQ(this=k, expression=v) for k, v in zip(expression.keys, expression.values)
         ]
     )
 
 
-def _parse_datediff(args: t.List) -> exp.DateDiff:
-    return exp.DateDiff(this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0))
-
-
-def _unix_to_time_sql(self: Snowflake.Generator, expression: exp.UnixToTime) -> str:
-    scale = expression.args.get("scale")
-    timestamp = self.sql(expression, "this")
-    if scale in (None, exp.UnixToTime.SECONDS):
-        return f"TO_TIMESTAMP({timestamp})"
-    if scale == exp.UnixToTime.MILLIS:
-        return f"TO_TIMESTAMP({timestamp}, 3)"
-    if scale == exp.UnixToTime.MICROS:
-        return f"TO_TIMESTAMP({timestamp} / 1000, 3)"
-    if scale == exp.UnixToTime.NANOS:
-        return f"TO_TIMESTAMP({timestamp}, 9)"
-
-    self.unsupported(f"Unsupported scale for timestamp: {scale}.")
-    return ""
-
-
-# https://docs.snowflake.com/en/sql-reference/functions/date_part.html
-# https://docs.snowflake.com/en/sql-reference/functions-date-time.html#label-supported-date-time-parts
-def _parse_date_part(self: Snowflake.Parser) -> t.Optional[exp.Expression]:
-    this = self._parse_var() or self._parse_type()
-
-    if not this:
-        return None
-
-    self._match(TokenType.COMMA)
-    expression = self._parse_bitwise()
-
-    name = this.name.upper()
-    if name.startswith("EPOCH"):
-        if name.startswith("EPOCH_MILLISECOND"):
-            scale = 10**3
-        elif name.startswith("EPOCH_MICROSECOND"):
-            scale = 10**6
-        elif name.startswith("EPOCH_NANOSECOND"):
-            scale = 10**9
-        else:
-            scale = None
-
-        ts = self.expression(exp.Cast, this=expression, to=exp.DataType.build("TIMESTAMP"))
-        to_unix: exp.Expression = self.expression(exp.TimeToUnix, this=ts)
-
-        if scale:
-            to_unix = exp.Mul(this=to_unix, expression=exp.Literal.number(scale))
-
-        return to_unix
-
-    return self.expression(exp.Extract, this=this, expression=expression)
+def _build_datediff(args: t.List) -> exp.DateDiff:
+    return exp.DateDiff(
+        this=seq_get(args, 2), expression=seq_get(args, 1), unit=_map_date_part(seq_get(args, 0))
+    )
 
 
 # https://docs.snowflake.com/en/sql-reference/functions/div0
-def _div0_to_if(args: t.List) -> exp.If:
+def _build_if_from_div0(args: t.List) -> exp.If:
     cond = exp.EQ(this=seq_get(args, 1), expression=exp.Literal.number(0))
     true = exp.Literal.number(0)
     false = exp.Div(this=seq_get(args, 0), expression=seq_get(args, 1))
@@ -152,13 +86,13 @@ def _div0_to_if(args: t.List) -> exp.If:
 
 
 # https://docs.snowflake.com/en/sql-reference/functions/zeroifnull
-def _zeroifnull_to_if(args: t.List) -> exp.If:
+def _build_if_from_zeroifnull(args: t.List) -> exp.If:
     cond = exp.Is(this=seq_get(args, 0), expression=exp.Null())
     return exp.If(this=cond, true=exp.Literal.number(0), false=seq_get(args, 0))
 
 
 # https://docs.snowflake.com/en/sql-reference/functions/zeroifnull
-def _nullifzero_to_if(args: t.List) -> exp.If:
+def _build_if_from_nullifzero(args: t.List) -> exp.If:
     cond = exp.EQ(this=seq_get(args, 0), expression=exp.Literal.number(0))
     return exp.If(this=cond, true=exp.Null(), false=seq_get(args, 0))
 
@@ -182,13 +116,13 @@ def _regexpilike_sql(self: Snowflake.Generator, expression: exp.RegexpILike) ->
     )
 
 
-def _parse_convert_timezone(args: t.List) -> t.Union[exp.Anonymous, exp.AtTimeZone]:
+def _build_convert_timezone(args: t.List) -> t.Union[exp.Anonymous, exp.AtTimeZone]:
     if len(args) == 3:
         return exp.Anonymous(this="CONVERT_TIMEZONE", expressions=args)
     return exp.AtTimeZone(this=seq_get(args, 1), zone=seq_get(args, 0))
 
 
-def _parse_regexp_replace(args: t.List) -> exp.RegexpReplace:
+def _build_regexp_replace(args: t.List) -> exp.RegexpReplace:
     regexp_replace = exp.RegexpReplace.from_arg_list(args)
 
     if not regexp_replace.args.get("replacement"):
@@ -204,6 +138,126 @@ def _show_parser(*args: t.Any, **kwargs: t.Any) -> t.Callable[[Snowflake.Parser]
     return _parse
 
 
+DATE_PART_MAPPING = {
+    "Y": "YEAR",
+    "YY": "YEAR",
+    "YYY": "YEAR",
+    "YYYY": "YEAR",
+    "YR": "YEAR",
+    "YEARS": "YEAR",
+    "YRS": "YEAR",
+    "MM": "MONTH",
+    "MON": "MONTH",
+    "MONS": "MONTH",
+    "MONTHS": "MONTH",
+    "D": "DAY",
+    "DD": "DAY",
+    "DAYS": "DAY",
+    "DAYOFMONTH": "DAY",
+    "WEEKDAY": "DAYOFWEEK",
+    "DOW": "DAYOFWEEK",
+    "DW": "DAYOFWEEK",
+    "WEEKDAY_ISO": "DAYOFWEEKISO",
+    "DOW_ISO": "DAYOFWEEKISO",
+    "DW_ISO": "DAYOFWEEKISO",
+    "YEARDAY": "DAYOFYEAR",
+    "DOY": "DAYOFYEAR",
+    "DY": "DAYOFYEAR",
+    "W": "WEEK",
+    "WK": "WEEK",
+    "WEEKOFYEAR": "WEEK",
+    "WOY": "WEEK",
+    "WY": "WEEK",
+    "WEEK_ISO": "WEEKISO",
+    "WEEKOFYEARISO": "WEEKISO",
+    "WEEKOFYEAR_ISO": "WEEKISO",
+    "Q": "QUARTER",
+    "QTR": "QUARTER",
+    "QTRS": "QUARTER",
+    "QUARTERS": "QUARTER",
+    "H": "HOUR",
+    "HH": "HOUR",
+    "HR": "HOUR",
+    "HOURS": "HOUR",
+    "HRS": "HOUR",
+    "M": "MINUTE",
+    "MI": "MINUTE",
+    "MIN": "MINUTE",
+    "MINUTES": "MINUTE",
+    "MINS": "MINUTE",
+    "S": "SECOND",
+    "SEC": "SECOND",
+    "SECONDS": "SECOND",
+    "SECS": "SECOND",
+    "MS": "MILLISECOND",
+    "MSEC": "MILLISECOND",
+    "MILLISECONDS": "MILLISECOND",
+    "US": "MICROSECOND",
+    "USEC": "MICROSECOND",
+    "MICROSECONDS": "MICROSECOND",
+    "NS": "NANOSECOND",
+    "NSEC": "NANOSECOND",
+    "NANOSEC": "NANOSECOND",
+    "NSECOND": "NANOSECOND",
+    "NSECONDS": "NANOSECOND",
+    "NANOSECS": "NANOSECOND",
+    "EPOCH": "EPOCH_SECOND",
+    "EPOCH_SECONDS": "EPOCH_SECOND",
+    "EPOCH_MILLISECONDS": "EPOCH_MILLISECOND",
+    "EPOCH_MICROSECONDS": "EPOCH_MICROSECOND",
+    "EPOCH_NANOSECONDS": "EPOCH_NANOSECOND",
+    "TZH": "TIMEZONE_HOUR",
+    "TZM": "TIMEZONE_MINUTE",
+}
+
+
+@t.overload
+def _map_date_part(part: exp.Expression) -> exp.Var:
+    pass
+
+
+@t.overload
+def _map_date_part(part: t.Optional[exp.Expression]) -> t.Optional[exp.Expression]:
+    pass
+
+
+def _map_date_part(part):
+    mapped = DATE_PART_MAPPING.get(part.name.upper()) if part else None
+    return exp.var(mapped) if mapped else part
+
+
+def _date_trunc_to_time(args: t.List) -> exp.DateTrunc | exp.TimestampTrunc:
+    trunc = date_trunc_to_time(args)
+    trunc.set("unit", _map_date_part(trunc.args["unit"]))
+    return trunc
+
+
+def _build_timestamp_from_parts(args: t.List) -> exp.Func:
+    if len(args) == 2:
+        # Other dialects don't have the TIMESTAMP_FROM_PARTS(date, time) concept,
+        # so we parse this into Anonymous for now instead of introducing complexity
+        return exp.Anonymous(this="TIMESTAMP_FROM_PARTS", expressions=args)
+
+    return exp.TimestampFromParts.from_arg_list(args)
+
+
+def _unqualify_unpivot_columns(expression: exp.Expression) -> exp.Expression:
+    """
+    Snowflake doesn't allow columns referenced in UNPIVOT to be qualified,
+    so we need to unqualify them.
+
+    Example:
+        >>> from sqlglot import parse_one
+        >>> expr = parse_one("SELECT * FROM m_sales UNPIVOT(sales FOR month IN (m_sales.jan, feb, mar, april))")
+        >>> print(_unqualify_unpivot_columns(expr).sql(dialect="snowflake"))
+        SELECT * FROM m_sales UNPIVOT(sales FOR month IN (jan, feb, mar, april))
+    """
+    if isinstance(expression, exp.Pivot) and expression.unpivot:
+        expression = transforms.unqualify_columns(expression)
+
+    return expression
+
+
 class Snowflake(Dialect):
     # https://docs.snowflake.com/en/sql-reference/identifiers-syntax
     NORMALIZATION_STRATEGY = NormalizationStrategy.UPPERCASE
@@ -211,6 +265,8 @@ class Snowflake(Dialect):
     TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
     SUPPORTS_USER_DEFINED_TYPES = False
     SUPPORTS_SEMI_ANTI_JOIN = False
+    PREFER_CTE_ALIAS_COLUMN = True
+    TABLESAMPLE_SIZE_IS_PERCENT = True
 
     TIME_MAPPING = {
         "YYYY": "%Y",
@@ -249,7 +305,7 @@ class Snowflake(Dialect):
             and isinstance(expression.parent, exp.Table)
             and expression.name.lower() == "dual"
         ):
-            return t.cast(E, expression)
+            return expression  # type: ignore
 
         return super().quote_identifier(expression, identify=identify)
 
@@ -275,48 +331,53 @@ class Snowflake(Dialect):
             "BITXOR": binary_from_function(exp.BitwiseXor),
             "BIT_XOR": binary_from_function(exp.BitwiseXor),
             "BOOLXOR": binary_from_function(exp.Xor),
-            "CONVERT_TIMEZONE": _parse_convert_timezone,
-            "DATE_TRUNC": date_trunc_to_time,
+            "CONVERT_TIMEZONE": _build_convert_timezone,
+            "DATE_TRUNC": _date_trunc_to_time,
             "DATEADD": lambda args: exp.DateAdd(
-                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
+                this=seq_get(args, 2),
+                expression=seq_get(args, 1),
+                unit=_map_date_part(seq_get(args, 0)),
             ),
-            "DATEDIFF": _parse_datediff,
-            "DIV0": _div0_to_if,
+            "DATEDIFF": _build_datediff,
+            "DIV0": _build_if_from_div0,
             "FLATTEN": exp.Explode.from_arg_list,
+            "GET_PATH": lambda args, dialect: exp.JSONExtract(
+                this=seq_get(args, 0), expression=dialect.to_json_path(seq_get(args, 1))
+            ),
             "IFF": exp.If.from_arg_list,
+            "LAST_DAY": lambda args: exp.LastDay(
+                this=seq_get(args, 0), unit=_map_date_part(seq_get(args, 1))
+            ),
             "LISTAGG": exp.GroupConcat.from_arg_list,
-            "NULLIFZERO": _nullifzero_to_if,
-            "OBJECT_CONSTRUCT": _parse_object_construct,
-            "REGEXP_REPLACE": _parse_regexp_replace,
+            "NULLIFZERO": _build_if_from_nullifzero,
+            "OBJECT_CONSTRUCT": _build_object_construct,
+            "REGEXP_REPLACE": _build_regexp_replace,
             "REGEXP_SUBSTR": exp.RegexpExtract.from_arg_list,
             "RLIKE": exp.RegexpLike.from_arg_list,
             "SQUARE": lambda args: exp.Pow(this=seq_get(args, 0), expression=exp.Literal.number(2)),
-            "TIMEDIFF": _parse_datediff,
-            "TIMESTAMPDIFF": _parse_datediff,
-            "TO_TIMESTAMP": _parse_to_timestamp,
+            "TIMEDIFF": _build_datediff,
+            "TIMESTAMPDIFF": _build_datediff,
+            "TIMESTAMPFROMPARTS": _build_timestamp_from_parts,
+            "TIMESTAMP_FROM_PARTS": _build_timestamp_from_parts,
+            "TO_TIMESTAMP": _build_to_timestamp,
             "TO_VARCHAR": exp.ToChar.from_arg_list,
-            "ZEROIFNULL": _zeroifnull_to_if,
+            "ZEROIFNULL": _build_if_from_zeroifnull,
         }
 
         FUNCTION_PARSERS = {
             **parser.Parser.FUNCTION_PARSERS,
-            "DATE_PART": _parse_date_part,
+            "DATE_PART": lambda self: self._parse_date_part(),
+            "OBJECT_CONSTRUCT_KEEP_NULL": lambda self: self._parse_json_object(),
         }
         FUNCTION_PARSERS.pop("TRIM")
 
-        COLUMN_OPERATORS = {
-            **parser.Parser.COLUMN_OPERATORS,
-            TokenType.COLON: lambda self, this, path: self.expression(
-                exp.Bracket, this=this, expressions=[path]
-            ),
-        }
-
         TIMESTAMPS = parser.Parser.TIMESTAMPS - {TokenType.TIME}
 
         RANGE_PARSERS = {
             **parser.Parser.RANGE_PARSERS,
             TokenType.LIKE_ANY: parser.binary_range_parser(exp.LikeAny),
             TokenType.ILIKE_ANY: parser.binary_range_parser(exp.ILikeAny),
+            TokenType.COLON: lambda self, this: self._parse_colon_get_path(this),
         }
 
         ALTER_PARSERS = {
@@ -342,8 +403,25 @@ class Snowflake(Dialect):
         }
 
         SHOW_PARSERS = {
+            "SCHEMAS": _show_parser("SCHEMAS"),
+            "TERSE SCHEMAS": _show_parser("SCHEMAS"),
+            "OBJECTS": _show_parser("OBJECTS"),
+            "TERSE OBJECTS": _show_parser("OBJECTS"),
+            "TABLES": _show_parser("TABLES"),
+            "TERSE TABLES": _show_parser("TABLES"),
+            "VIEWS": _show_parser("VIEWS"),
+            "TERSE VIEWS": _show_parser("VIEWS"),
             "PRIMARY KEYS": _show_parser("PRIMARY KEYS"),
             "TERSE PRIMARY KEYS": _show_parser("PRIMARY KEYS"),
+            "IMPORTED KEYS": _show_parser("IMPORTED KEYS"),
+            "TERSE IMPORTED KEYS": _show_parser("IMPORTED KEYS"),
+            "UNIQUE KEYS": _show_parser("UNIQUE KEYS"),
+            "TERSE UNIQUE KEYS": _show_parser("UNIQUE KEYS"),
+            "SEQUENCES": _show_parser("SEQUENCES"),
+            "TERSE SEQUENCES": _show_parser("SEQUENCES"),
+            "COLUMNS": _show_parser("COLUMNS"),
+            "USERS": _show_parser("USERS"),
+            "TERSE USERS": _show_parser("USERS"),
         }
 
         STAGED_FILE_SINGLE_TOKENS = {
@@ -351,8 +429,83 @@ class Snowflake(Dialect):
             TokenType.MOD,
             TokenType.SLASH,
         }
+
         FLATTEN_COLUMNS = ["SEQ", "KEY", "PATH", "INDEX", "VALUE", "THIS"]
 
+        SCHEMA_KINDS = {"OBJECTS", "TABLES", "VIEWS", "SEQUENCES", "UNIQUE KEYS", "IMPORTED KEYS"}
+
+        def _parse_colon_get_path(
+            self: parser.Parser, this: t.Optional[exp.Expression]
+        ) -> t.Optional[exp.Expression]:
+            while True:
+                path = self._parse_bitwise() or self._parse_var(any_token=True)
+
+                # The cast :: operator has a lower precedence than the extraction operator :, so
+                # we rearrange the AST appropriately to avoid casting the 2nd argument of GET_PATH
+                if isinstance(path, exp.Cast):
+                    target_type = path.to
+                    path = path.this
+                else:
+                    target_type = None
+
+                if isinstance(path, exp.Expression):
+                    path = exp.Literal.string(path.sql(dialect="snowflake"))
+
+                # The extraction operator : is left-associative
+                this = self.expression(
+                    exp.JSONExtract, this=this, expression=self.dialect.to_json_path(path)
+                )
+
+                if target_type:
+                    this = exp.cast(this, target_type)
+
+                if not self._match(TokenType.COLON):
+                    break
+
+            return self._parse_range(this)
+
+        # https://docs.snowflake.com/en/sql-reference/functions/date_part.html
+        # https://docs.snowflake.com/en/sql-reference/functions-date-time.html#label-supported-date-time-parts
+        def _parse_date_part(self: Snowflake.Parser) -> t.Optional[exp.Expression]:
+            this = self._parse_var() or self._parse_type()
+
+            if not this:
+                return None
+
+            self._match(TokenType.COMMA)
+            expression = self._parse_bitwise()
+            this = _map_date_part(this)
+            name = this.name.upper()
+
+            if name.startswith("EPOCH"):
+                if name == "EPOCH_MILLISECOND":
+                    scale = 10**3
+                elif name == "EPOCH_MICROSECOND":
+                    scale = 10**6
+                elif name == "EPOCH_NANOSECOND":
+                    scale = 10**9
+                else:
+                    scale = None
+
+                ts = self.expression(exp.Cast, this=expression, to=exp.DataType.build("TIMESTAMP"))
+                to_unix: exp.Expression = self.expression(exp.TimeToUnix, this=ts)
+
+                if scale:
+                    to_unix = exp.Mul(this=to_unix, expression=exp.Literal.number(scale))
+
+                return to_unix
+
+            return self.expression(exp.Extract, this=this, expression=expression)
+
+        def _parse_bracket_key_value(self, is_map: bool = False) -> t.Optional[exp.Expression]:
+            if is_map:
+                # Keys are strings in Snowflake's objects, see also:
+                # - https://docs.snowflake.com/en/sql-reference/data-types-semistructured
+                # - https://docs.snowflake.com/en/sql-reference/functions/object_construct
+                return self._parse_slice(self._parse_string())
+
+            return self._parse_slice(self._parse_alias(self._parse_conjunction(), explicit=True))
+
         def _parse_lateral(self) -> t.Optional[exp.Lateral]:
             lateral = super()._parse_lateral()
             if not lateral:
@@ -391,7 +544,9 @@ class Snowflake(Dialect):
 
             return table
 
-        def _parse_table_parts(self, schema: bool = False) -> exp.Table:
+        def _parse_table_parts(
+            self, schema: bool = False, is_db_reference: bool = False, wildcard: bool = False
+        ) -> exp.Table:
             # https://docs.snowflake.com/en/user-guide/querying-stage
             if self._match(TokenType.STRING, advance=False):
                 table = self._parse_string()
@@ -407,7 +562,9 @@ class Snowflake(Dialect):
                 self._match(TokenType.L_PAREN)
                 while self._curr and not self._match(TokenType.R_PAREN):
                     if self._match_text_seq("FILE_FORMAT", "=>"):
-                        file_format = self._parse_string() or super()._parse_table_parts()
+                        file_format = self._parse_string() or super()._parse_table_parts(
+                            is_db_reference=is_db_reference
+                        )
                     elif self._match_text_seq("PATTERN", "=>"):
                         pattern = self._parse_string()
                     else:
@@ -417,7 +574,7 @@ class Snowflake(Dialect):
 
                 table = self.expression(exp.Table, this=table, format=file_format, pattern=pattern)
             else:
-                table = super()._parse_table_parts(schema=schema)
+                table = super()._parse_table_parts(schema=schema, is_db_reference=is_db_reference)
 
             return self._parse_at_before(table)
 
@@ -440,18 +597,39 @@ class Snowflake(Dialect):
             scope = None
             scope_kind = None
 
+            # will identity SHOW TERSE SCHEMAS but not SHOW TERSE PRIMARY KEYS
+            # which is syntactically valid but has no effect on the output
+            terse = self._tokens[self._index - 2].text.upper() == "TERSE"
+
+            history = self._match_text_seq("HISTORY")
+
+            like = self._parse_string() if self._match(TokenType.LIKE) else None
+
             if self._match(TokenType.IN):
                 if self._match_text_seq("ACCOUNT"):
                     scope_kind = "ACCOUNT"
                 elif self._match_set(self.DB_CREATABLES):
-                    scope_kind = self._prev.text
+                    scope_kind = self._prev.text.upper()
                     if self._curr:
-                        scope = self._parse_table()
+                        scope = self._parse_table_parts()
                 elif self._curr:
-                    scope_kind = "TABLE"
-                    scope = self._parse_table()
+                    scope_kind = "SCHEMA" if this in self.SCHEMA_KINDS else "TABLE"
+                    scope = self._parse_table_parts()
 
-            return self.expression(exp.Show, this=this, scope=scope, scope_kind=scope_kind)
+            return self.expression(
+                exp.Show,
+                **{
+                    "terse": terse,
+                    "this": this,
+                    "history": history,
+                    "like": like,
+                    "scope": scope,
+                    "scope_kind": scope_kind,
+                    "starts_with": self._match_text_seq("STARTS", "WITH") and self._parse_string(),
+                    "limit": self._parse_limit(),
+                    "from": self._parse_string() if self._match(TokenType.FROM) else None,
+                },
+            )
 
         def _parse_alter_table_swap(self) -> exp.SwapTable:
             self._match_text_seq("WITH")
@@ -489,8 +667,13 @@ class Snowflake(Dialect):
             "MINUS": TokenType.EXCEPT,
             "NCHAR VARYING": TokenType.VARCHAR,
             "PUT": TokenType.COMMAND,
+            "REMOVE": TokenType.COMMAND,
             "RENAME": TokenType.REPLACE,
+            "RM": TokenType.COMMAND,
             "SAMPLE": TokenType.TABLE_SAMPLE,
+            "SQL_DOUBLE": TokenType.DOUBLE,
+            "SQL_VARCHAR": TokenType.VARCHAR,
+            "STORAGE INTEGRATION": TokenType.STORAGE_INTEGRATION,
             "TIMESTAMP_LTZ": TokenType.TIMESTAMPLTZ,
             "TIMESTAMP_NTZ": TokenType.TIMESTAMP,
             "TIMESTAMP_TZ": TokenType.TIMESTAMPTZ,
@@ -518,6 +701,8 @@ class Snowflake(Dialect):
         SUPPORTS_TABLE_COPY = False
         COLLATE_IS_FUNC = True
         LIMIT_ONLY_LITERALS = True
+        JSON_KEY_VALUE_PAIR_SEP = ","
+        INSERT_OVERWRITE = " OVERWRITE INTO"
 
         TRANSFORMS = {
             **generator.Generator.TRANSFORMS,
@@ -540,11 +725,20 @@ class Snowflake(Dialect):
             exp.DayOfYear: rename_func("DAYOFYEAR"),
             exp.Explode: rename_func("FLATTEN"),
             exp.Extract: rename_func("DATE_PART"),
+            exp.FromTimeZone: lambda self, e: self.func(
+                "CONVERT_TIMEZONE", e.args.get("zone"), "'UTC'", e.this
+            ),
             exp.GenerateSeries: lambda self, e: self.func(
                 "ARRAY_GENERATE_RANGE", e.args["start"], e.args["end"] + 1, e.args.get("step")
             ),
             exp.GroupConcat: rename_func("LISTAGG"),
             exp.If: if_sql(name="IFF", false_value="NULL"),
+            exp.JSONExtract: lambda self, e: self.func("GET_PATH", e.this, e.expression),
+            exp.JSONExtractScalar: lambda self, e: self.func(
+                "JSON_EXTRACT_PATH_TEXT", e.this, e.expression
+            ),
+            exp.JSONObject: lambda self, e: self.func("OBJECT_CONSTRUCT_KEEP_NULL", *e.expressions),
+            exp.JSONPathRoot: lambda *_: "",
             exp.LogicalAnd: rename_func("BOOLAND_AGG"),
             exp.LogicalOr: rename_func("BOOLOR_AGG"),
             exp.Map: lambda self, e: var_map_sql(self, e, "OBJECT_CONSTRUCT"),
@@ -557,6 +751,7 @@ class Snowflake(Dialect):
             exp.PercentileDisc: transforms.preprocess(
                 [transforms.add_within_group_for_percentiles]
             ),
+            exp.Pivot: transforms.preprocess([_unqualify_unpivot_columns]),
             exp.RegexpILike: _regexpilike_sql,
             exp.Rand: rename_func("RANDOM"),
             exp.Select: transforms.preprocess(
@@ -572,12 +767,11 @@ class Snowflake(Dialect):
             exp.StrPosition: lambda self, e: self.func(
                 "POSITION", e.args.get("substr"), e.this, e.args.get("position")
             ),
-            exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
-            exp.Struct: lambda self, e: self.func(
-                "OBJECT_CONSTRUCT",
-                *(arg for expression in e.expressions for arg in expression.flatten()),
-            ),
+            exp.StrToTime: lambda self, e: self.func("TO_TIMESTAMP", e.this, self.format_time(e)),
             exp.Stuff: rename_func("INSERT"),
+            exp.TimestampDiff: lambda self, e: self.func(
+                "TIMESTAMPDIFF", e.unit, e.expression, e.this
+            ),
             exp.TimestampTrunc: timestamptrunc_sql,
             exp.TimeStrToTime: timestrtotime_sql,
             exp.TimeToStr: lambda self, e: self.func(
@@ -589,13 +783,18 @@ class Snowflake(Dialect):
             exp.Trim: lambda self, e: self.func("TRIM", e.this, e.expression),
             exp.TsOrDsAdd: date_delta_sql("DATEADD", cast=True),
             exp.TsOrDsDiff: date_delta_sql("DATEDIFF"),
-            exp.TsOrDsToDate: ts_or_ds_to_date_sql("snowflake"),
-            exp.UnixToTime: _unix_to_time_sql,
+            exp.UnixToTime: rename_func("TO_TIMESTAMP"),
             exp.VarMap: lambda self, e: var_map_sql(self, e, "OBJECT_CONSTRUCT"),
             exp.WeekOfYear: rename_func("WEEKOFYEAR"),
             exp.Xor: rename_func("BOOLXOR"),
         }
 
+        SUPPORTED_JSON_PATH_PARTS = {
+            exp.JSONPathKey,
+            exp.JSONPathRoot,
+            exp.JSONPathSubscript,
+        }
+
         TYPE_MAPPING = {
             **generator.Generator.TYPE_MAPPING,
             exp.DataType.Type.TIMESTAMP: "TIMESTAMPNTZ",
@@ -612,6 +811,14 @@ class Snowflake(Dialect):
             exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
         }
 
+        def timestampfromparts_sql(self, expression: exp.TimestampFromParts) -> str:
+            milli = expression.args.get("milli")
+            if milli is not None:
+                milli_to_nano = milli.pop() * exp.Literal.number(1000000)
+                expression.set("nano", milli_to_nano)
+
+            return rename_func("TIMESTAMP_FROM_PARTS")(self, expression)
+
         def trycast_sql(self, expression: exp.TryCast) -> str:
             value = expression.this
 
@@ -657,6 +864,11 @@ class Snowflake(Dialect):
             return f"{explode}{alias}"
 
         def show_sql(self, expression: exp.Show) -> str:
+            terse = "TERSE " if expression.args.get("terse") else ""
+            history = " HISTORY" if expression.args.get("history") else ""
+            like = self.sql(expression, "like")
+            like = f" LIKE {like}" if like else ""
+
             scope = self.sql(expression, "scope")
             scope = f" {scope}" if scope else ""
 
@@ -664,7 +876,17 @@ class Snowflake(Dialect):
             if scope_kind:
                 scope_kind = f" IN {scope_kind}"
 
-            return f"SHOW {expression.name}{scope_kind}{scope}"
+            starts_with = self.sql(expression, "starts_with")
+            if starts_with:
+                starts_with = f" STARTS WITH {starts_with}"
+
+            limit = self.sql(expression, "limit")
+
+            from_ = self.sql(expression, "from")
+            if from_:
+                from_ = f" FROM {from_}"
+
+            return f"SHOW {terse}{expression.name}{history}{like}{scope_kind}{scope}{starts_with}{limit}{from_}"
 
         def regexpextract_sql(self, expression: exp.RegexpExtract) -> str:
             # Other dialects don't support all of the following parameters, so we need to
@@ -685,12 +907,12 @@ class Snowflake(Dialect):
             )
 
         def except_op(self, expression: exp.Except) -> str:
-            if not expression.args.get("distinct", False):
+            if not expression.args.get("distinct"):
                 self.unsupported("EXCEPT with All is not supported in Snowflake")
             return super().except_op(expression)
 
         def intersect_op(self, expression: exp.Intersect) -> str:
-            if not expression.args.get("distinct", False):
+            if not expression.args.get("distinct"):
                 self.unsupported("INTERSECT with All is not supported in Snowflake")
             return super().intersect_op(expression)
 
@@ -718,3 +940,22 @@ class Snowflake(Dialect):
 
         def with_properties(self, properties: exp.Properties) -> str:
             return self.properties(properties, wrapped=False, prefix=self.seg(""), sep=" ")
+
+        def cluster_sql(self, expression: exp.Cluster) -> str:
+            return f"CLUSTER BY ({self.expressions(expression, flat=True)})"
+
+        def struct_sql(self, expression: exp.Struct) -> str:
+            keys = []
+            values = []
+
+            for i, e in enumerate(expression.expressions):
+                if isinstance(e, exp.PropertyEQ):
+                    keys.append(
+                        exp.Literal.string(e.name) if isinstance(e.this, exp.Identifier) else e.this
+                    )
+                    values.append(e.expression)
+                else:
+                    keys.append(exp.Literal.string(f"_{i}"))
+                    values.append(e)
+
+            return self.func("OBJECT_CONSTRUCT", *flatten(zip(keys, values)))
