@@ -9,41 +9,55 @@ from sqlglot.tokens import TokenType
 
 class Teradata(Dialect):
     SUPPORTS_SEMI_ANTI_JOIN = False
+    TYPED_DIVISION = True
 
     TIME_MAPPING = {
-        "Y": "%Y",
-        "YYYY": "%Y",
         "YY": "%y",
-        "MMMM": "%B",
-        "MMM": "%b",
-        "DD": "%d",
-        "D": "%-d",
-        "HH": "%H",
-        "H": "%-H",
-        "MM": "%M",
+        "Y4": "%Y",
+        "YYYY": "%Y",
+        "M4": "%B",
+        "M3": "%b",
         "M": "%-M",
-        "SS": "%S",
+        "MI": "%M",
+        "MM": "%m",
+        "MMM": "%b",
+        "MMMM": "%B",
+        "D": "%-d",
+        "DD": "%d",
+        "D3": "%j",
+        "DDD": "%j",
+        "H": "%-H",
+        "HH": "%H",
+        "HH24": "%H",
         "S": "%-S",
+        "SS": "%S",
         "SSSSSS": "%f",
         "E": "%a",
         "EE": "%a",
+        "E3": "%a",
+        "E4": "%A",
         "EEE": "%a",
         "EEEE": "%A",
     }
 
     class Tokenizer(tokens.Tokenizer):
         # https://docs.teradata.com/r/Teradata-Database-SQL-Functions-Operators-Expressions-and-Predicates/March-2017/Comparison-Operators-and-Functions/Comparison-Operators/ANSI-Compliance
+        # https://docs.teradata.com/r/SQL-Functions-Operators-Expressions-and-Predicates/June-2017/Arithmetic-Trigonometric-Hyperbolic-Operators/Functions
         KEYWORDS = {
             **tokens.Tokenizer.KEYWORDS,
+            "**": TokenType.DSTAR,
             "^=": TokenType.NEQ,
             "BYTEINT": TokenType.SMALLINT,
             "COLLECT": TokenType.COMMAND,
+            "DEL": TokenType.DELETE,
+            "EQ": TokenType.EQ,
             "GE": TokenType.GTE,
             "GT": TokenType.GT,
             "HELP": TokenType.COMMAND,
             "INS": TokenType.INSERT,
             "LE": TokenType.LTE,
             "LT": TokenType.LT,
+            "MINUS": TokenType.EXCEPT,
             "MOD": TokenType.MOD,
             "NE": TokenType.NEQ,
             "NOT=": TokenType.NEQ,
@@ -51,6 +65,7 @@ class Teradata(Dialect):
             "SEL": TokenType.SELECT,
             "ST_GEOMETRY": TokenType.GEOMETRY,
             "TOP": TokenType.TOP,
+            "UPD": TokenType.UPDATE,
         }
 
         # Teradata does not support % as a modulo operator
@@ -59,6 +74,7 @@ class Teradata(Dialect):
 
     class Parser(parser.Parser):
         TABLESAMPLE_CSV = True
+        VALUES_FOLLOWED_BY_PAREN = False
 
         CHARSET_TRANSLATORS = {
             "GRAPHIC_TO_KANJISJIS",
@@ -108,10 +124,16 @@ class Teradata(Dialect):
 
         FUNCTION_PARSERS = {
             **parser.Parser.FUNCTION_PARSERS,
+            # https://docs.teradata.com/r/SQL-Functions-Operators-Expressions-and-Predicates/June-2017/Data-Type-Conversions/TRYCAST
+            "TRYCAST": parser.Parser.FUNCTION_PARSERS["TRY_CAST"],
             "RANGE_N": lambda self: self._parse_rangen(),
             "TRANSLATE": lambda self: self._parse_translate(self.STRICT_CAST),
         }
 
+        EXPONENT = {
+            TokenType.DSTAR: exp.Pow,
+        }
+
         def _parse_translate(self, strict: bool) -> exp.Expression:
             this = self._parse_conjunction()
 
@@ -154,6 +176,9 @@ class Teradata(Dialect):
         JOIN_HINTS = False
         TABLE_HINTS = False
         QUERY_HINTS = False
+        TABLESAMPLE_KEYWORDS = "SAMPLE"
+        LAST_DAY_SUPPORTS_DATE_PART = False
+        CAN_IMPLEMENT_ARRAY_ANY = True
 
         TYPE_MAPPING = {
             **generator.Generator.TYPE_MAPPING,
@@ -171,18 +196,34 @@ class Teradata(Dialect):
             **generator.Generator.TRANSFORMS,
             exp.ArgMax: rename_func("MAX_BY"),
             exp.ArgMin: rename_func("MIN_BY"),
+            exp.ArraySize: rename_func("CARDINALITY"),
             exp.Max: max_or_greatest,
             exp.Min: min_or_least,
+            exp.Pow: lambda self, e: self.binary(e, "**"),
             exp.Select: transforms.preprocess(
                 [transforms.eliminate_distinct_on, transforms.eliminate_semi_and_anti_joins]
             ),
-            exp.StrToDate: lambda self, e: f"CAST({self.sql(e, 'this')} AS DATE FORMAT {self.format_time(e)})",
+            exp.StrToDate: lambda self,
+            e: f"CAST({self.sql(e, 'this')} AS DATE FORMAT {self.format_time(e)})",
             exp.ToChar: lambda self, e: self.function_fallback_sql(e),
             exp.Use: lambda self, e: f"DATABASE {self.sql(e, 'this')}",
         }
 
+        def cast_sql(self, expression: exp.Cast, safe_prefix: t.Optional[str] = None) -> str:
+            if expression.to.this == exp.DataType.Type.UNKNOWN and expression.args.get("format"):
+                # We don't actually want to print the unknown type in CAST(<value> AS FORMAT <format>)
+                expression.to.pop()
+
+            return super().cast_sql(expression, safe_prefix=safe_prefix)
+
+        def trycast_sql(self, expression: exp.TryCast) -> str:
+            return self.cast_sql(expression, safe_prefix="TRY")
+
         def tablesample_sql(
-            self, expression: exp.TableSample, seed_prefix: str = "SEED", sep=" AS "
+            self,
+            expression: exp.TableSample,
+            sep: str = " AS ",
+            tablesample_keyword: t.Optional[str] = None,
         ) -> str:
             return f"{self.sql(expression, 'this')} SAMPLE {self.expressions(expression)}"
 
