@@ -13,12 +13,11 @@ from sqlglot.dialects.dialect import (
     generatedasidentitycolumnconstraint_sql,
     max_or_greatest,
     min_or_least,
-    parse_date_delta,
+    build_date_delta,
     rename_func,
     timestrtotime_sql,
-    ts_or_ds_to_date_sql,
+    trim_sql,
 )
-from sqlglot.expressions import DataType
 from sqlglot.helper import seq_get
 from sqlglot.time import format_time
 from sqlglot.tokens import TokenType
@@ -63,11 +62,49 @@ DEFAULT_START_DATE = datetime.date(1900, 1, 1)
 
 BIT_TYPES = {exp.EQ, exp.NEQ, exp.Is, exp.In, exp.Select, exp.Alias}
 
+# Unsupported options:
+# - OPTIMIZE FOR ( @variable_name { UNKNOWN | = <literal_constant> } [ , ...n ] )
+# - TABLE HINT
+OPTIONS: parser.OPTIONS_TYPE = {
+    **dict.fromkeys(
+        (
+            "DISABLE_OPTIMIZED_PLAN_FORCING",
+            "FAST",
+            "IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX",
+            "LABEL",
+            "MAXDOP",
+            "MAXRECURSION",
+            "MAX_GRANT_PERCENT",
+            "MIN_GRANT_PERCENT",
+            "NO_PERFORMANCE_SPOOL",
+            "QUERYTRACEON",
+            "RECOMPILE",
+        ),
+        tuple(),
+    ),
+    "CONCAT": ("UNION",),
+    "DISABLE": ("EXTERNALPUSHDOWN", "SCALEOUTEXECUTION"),
+    "EXPAND": ("VIEWS",),
+    "FORCE": ("EXTERNALPUSHDOWN", "ORDER", "SCALEOUTEXECUTION"),
+    "HASH": ("GROUP", "JOIN", "UNION"),
+    "KEEP": ("PLAN",),
+    "KEEPFIXED": ("PLAN",),
+    "LOOP": ("JOIN",),
+    "MERGE": ("JOIN", "UNION"),
+    "OPTIMIZE": (("FOR", "UNKNOWN"),),
+    "ORDER": ("GROUP",),
+    "PARAMETERIZATION": ("FORCED", "SIMPLE"),
+    "ROBUST": ("PLAN",),
+    "USE": ("PLAN",),
+}
 
-def _format_time_lambda(
+OPTIONS_THAT_REQUIRE_EQUAL = ("MAX_GRANT_PERCENT", "MIN_GRANT_PERCENT", "LABEL")
+
+
+def _build_formatted_time(
     exp_class: t.Type[E], full_format_mapping: t.Optional[bool] = None
 ) -> t.Callable[[t.List], E]:
-    def _format_time(args: t.List) -> E:
+    def _builder(args: t.List) -> E:
         assert len(args) == 2
 
         return exp_class(
@@ -75,17 +112,19 @@ def _format_time_lambda(
             format=exp.Literal.string(
                 format_time(
                     args[0].name.lower(),
-                    {**TSQL.TIME_MAPPING, **FULL_FORMAT_TIME_MAPPING}
-                    if full_format_mapping
-                    else TSQL.TIME_MAPPING,
+                    (
+                        {**TSQL.TIME_MAPPING, **FULL_FORMAT_TIME_MAPPING}
+                        if full_format_mapping
+                        else TSQL.TIME_MAPPING
+                    ),
                 )
             ),
         )
 
-    return _format_time
+    return _builder
 
 
-def _parse_format(args: t.List) -> exp.Expression:
+def _build_format(args: t.List) -> exp.NumberToStr | exp.TimeToStr:
     this = seq_get(args, 0)
     fmt = seq_get(args, 1)
     culture = seq_get(args, 2)
@@ -105,21 +144,20 @@ def _parse_format(args: t.List) -> exp.Expression:
     return exp.TimeToStr(this=this, format=fmt, culture=culture)
 
 
-def _parse_eomonth(args: t.List) -> exp.Expression:
-    date = seq_get(args, 0)
+def _build_eomonth(args: t.List) -> exp.LastDay:
+    date = exp.TsOrDsToDate(this=seq_get(args, 0))
     month_lag = seq_get(args, 1)
-    unit = DATE_DELTA_INTERVAL.get("month")
 
     if month_lag is None:
-        return exp.LastDateOfMonth(this=date)
+        this: exp.Expression = date
+    else:
+        unit = DATE_DELTA_INTERVAL.get("month")
+        this = exp.DateAdd(this=date, expression=month_lag, unit=unit and exp.var(unit))
 
-    # Remove month lag argument in parser as its compared with the number of arguments of the resulting class
-    args.remove(month_lag)
-
-    return exp.LastDateOfMonth(this=exp.DateAdd(this=date, expression=month_lag, unit=unit))
+    return exp.LastDay(this=this)
 
 
-def _parse_hashbytes(args: t.List) -> exp.Expression:
+def _build_hashbytes(args: t.List) -> exp.Expression:
     kind, data = args
     kind = kind.name.upper() if kind.is_string else ""
 
@@ -137,26 +175,27 @@ def _parse_hashbytes(args: t.List) -> exp.Expression:
     return exp.func("HASHBYTES", *args)
 
 
-DATEPART_ONLY_FORMATS = {"dw", "hour", "quarter"}
+DATEPART_ONLY_FORMATS = {"DW", "HOUR", "QUARTER"}
 
 
 def _format_sql(self: TSQL.Generator, expression: exp.NumberToStr | exp.TimeToStr) -> str:
-    fmt = (
-        expression.args["format"]
-        if isinstance(expression, exp.NumberToStr)
-        else exp.Literal.string(
-            format_time(
-                expression.text("format"),
-                t.cast(t.Dict[str, str], TSQL.INVERSE_TIME_MAPPING),
-            )
-        )
-    )
+    fmt = expression.args["format"]
 
-    # There is no format for "quarter"
-    if fmt.name.lower() in DATEPART_ONLY_FORMATS:
-        return self.func("DATEPART", fmt.name, expression.this)
+    if not isinstance(expression, exp.NumberToStr):
+        if fmt.is_string:
+            mapped_fmt = format_time(fmt.name, TSQL.INVERSE_TIME_MAPPING)
 
-    return self.func("FORMAT", expression.this, fmt, expression.args.get("culture"))
+            name = (mapped_fmt or "").upper()
+            if name in DATEPART_ONLY_FORMATS:
+                return self.func("DATEPART", name, expression.this)
+
+            fmt_sql = self.sql(exp.Literal.string(mapped_fmt))
+        else:
+            fmt_sql = self.format_time(expression) or self.sql(fmt)
+    else:
+        fmt_sql = self.sql(fmt)
+
+    return self.func("FORMAT", expression.this, fmt_sql, expression.args.get("culture"))
 
 
 def _string_agg_sql(self: TSQL.Generator, expression: exp.GroupConcat) -> str:
@@ -177,10 +216,10 @@ def _string_agg_sql(self: TSQL.Generator, expression: exp.GroupConcat) -> str:
     return f"STRING_AGG({self.format_args(this, separator)}){order}"
 
 
-def _parse_date_delta(
+def _build_date_delta(
     exp_class: t.Type[E], unit_mapping: t.Optional[t.Dict[str, str]] = None
 ) -> t.Callable[[t.List], E]:
-    def inner_func(args: t.List) -> E:
+    def _builder(args: t.List) -> E:
         unit = seq_get(args, 0)
         if unit and unit_mapping:
             unit = exp.var(unit_mapping.get(unit.name.lower(), unit.name))
@@ -202,7 +241,7 @@ def _parse_date_delta(
             unit=unit,
         )
 
-    return inner_func
+    return _builder
 
 
 def qualify_derived_table_outputs(expression: exp.Expression) -> exp.Expression:
@@ -219,19 +258,17 @@ def qualify_derived_table_outputs(expression: exp.Expression) -> exp.Expression:
         # We keep track of the unaliased column projection indexes instead of the expressions
         # themselves, because the latter are going to be replaced by new nodes when the aliases
         # are added and hence we won't be able to reach these newly added Alias parents
-        subqueryable = expression.this
+        query = expression.this
         unaliased_column_indexes = (
-            i
-            for i, c in enumerate(subqueryable.selects)
-            if isinstance(c, exp.Column) and not c.alias
+            i for i, c in enumerate(query.selects) if isinstance(c, exp.Column) and not c.alias
         )
 
-        qualify_outputs(subqueryable)
+        qualify_outputs(query)
 
         # Preserve the quoting information of columns for newly added Alias nodes
-        subqueryable_selects = subqueryable.selects
+        query_selects = query.selects
         for select_index in unaliased_column_indexes:
-            alias = subqueryable_selects[select_index]
+            alias = query_selects[select_index]
             column = alias.this
             if isinstance(column.this, exp.Identifier):
                 alias.args["alias"].set("quoted", column.this.quoted)
@@ -239,6 +276,58 @@ def qualify_derived_table_outputs(expression: exp.Expression) -> exp.Expression:
     return expression
 
 
+# https://learn.microsoft.com/en-us/sql/t-sql/functions/datetimefromparts-transact-sql?view=sql-server-ver16#syntax
+def _build_datetimefromparts(args: t.List) -> exp.TimestampFromParts:
+    return exp.TimestampFromParts(
+        year=seq_get(args, 0),
+        month=seq_get(args, 1),
+        day=seq_get(args, 2),
+        hour=seq_get(args, 3),
+        min=seq_get(args, 4),
+        sec=seq_get(args, 5),
+        milli=seq_get(args, 6),
+    )
+
+
+# https://learn.microsoft.com/en-us/sql/t-sql/functions/timefromparts-transact-sql?view=sql-server-ver16#syntax
+def _build_timefromparts(args: t.List) -> exp.TimeFromParts:
+    return exp.TimeFromParts(
+        hour=seq_get(args, 0),
+        min=seq_get(args, 1),
+        sec=seq_get(args, 2),
+        fractions=seq_get(args, 3),
+        precision=seq_get(args, 4),
+    )
+
+
+def _build_with_arg_as_text(
+    klass: t.Type[exp.Expression],
+) -> t.Callable[[t.List[exp.Expression]], exp.Expression]:
+    def _parse(args: t.List[exp.Expression]) -> exp.Expression:
+        this = seq_get(args, 0)
+
+        if this and not this.is_string:
+            this = exp.cast(this, exp.DataType.Type.TEXT)
+
+        expression = seq_get(args, 1)
+        kwargs = {"this": this}
+
+        if expression:
+            kwargs["expression"] = expression
+
+        return klass(**kwargs)
+
+    return _parse
+
+
+def _json_extract_sql(
+    self: TSQL.Generator, expression: exp.JSONExtract | exp.JSONExtractScalar
+) -> str:
+    json_query = self.func("JSON_QUERY", expression.this, expression.expression)
+    json_value = self.func("JSON_VALUE", expression.this, expression.expression)
+    return self.func("ISNULL", json_query, json_value)
+
+
 class TSQL(Dialect):
     NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_INSENSITIVE
     TIME_FORMAT = "'yyyy-mm-dd hh:mm:ss'"
@@ -352,7 +441,7 @@ class TSQL(Dialect):
     }
 
     class Tokenizer(tokens.Tokenizer):
-        IDENTIFIERS = ['"', ("[", "]")]
+        IDENTIFIERS = [("[", "]"), '"']
         QUOTES = ["'", '"']
         HEX_STRINGS = [("0x", ""), ("0X", "")]
         VAR_SINGLE_TOKENS = {"@", "$", "#"}
@@ -362,10 +451,10 @@ class TSQL(Dialect):
             "DATETIME2": TokenType.DATETIME,
             "DATETIMEOFFSET": TokenType.TIMESTAMPTZ,
             "DECLARE": TokenType.COMMAND,
+            "EXEC": TokenType.COMMAND,
             "IMAGE": TokenType.IMAGE,
             "MONEY": TokenType.MONEY,
             "NTEXT": TokenType.TEXT,
-            "NVARCHAR(MAX)": TokenType.TEXT,
             "PRINT": TokenType.COMMAND,
             "PROC": TokenType.PROCEDURE,
             "REAL": TokenType.FLOAT,
@@ -376,15 +465,24 @@ class TSQL(Dialect):
             "TOP": TokenType.TOP,
             "UNIQUEIDENTIFIER": TokenType.UNIQUEIDENTIFIER,
             "UPDATE STATISTICS": TokenType.COMMAND,
-            "VARCHAR(MAX)": TokenType.TEXT,
             "XML": TokenType.XML,
             "OUTPUT": TokenType.RETURNING,
             "SYSTEM_USER": TokenType.CURRENT_USER,
             "FOR SYSTEM_TIME": TokenType.TIMESTAMP_SNAPSHOT,
+            "OPTION": TokenType.OPTION,
         }
 
     class Parser(parser.Parser):
         SET_REQUIRES_ASSIGNMENT_DELIMITER = False
+        LOG_DEFAULTS_TO_LN = True
+        ALTER_TABLE_ADD_REQUIRED_FOR_EACH_COLUMN = False
+        STRING_ALIASES = True
+        NO_PAREN_IF_COMMANDS = False
+
+        QUERY_MODIFIER_PARSERS = {
+            **parser.Parser.QUERY_MODIFIER_PARSERS,
+            TokenType.OPTION: lambda self: ("options", self._parse_options()),
+        }
 
         FUNCTIONS = {
             **parser.Parser.FUNCTIONS,
@@ -393,39 +491,31 @@ class TSQL(Dialect):
                 substr=seq_get(args, 0),
                 position=seq_get(args, 2),
             ),
-            "DATEADD": parse_date_delta(exp.DateAdd, unit_mapping=DATE_DELTA_INTERVAL),
-            "DATEDIFF": _parse_date_delta(exp.DateDiff, unit_mapping=DATE_DELTA_INTERVAL),
-            "DATENAME": _format_time_lambda(exp.TimeToStr, full_format_mapping=True),
-            "DATEPART": _format_time_lambda(exp.TimeToStr),
-            "EOMONTH": _parse_eomonth,
-            "FORMAT": _parse_format,
+            "DATEADD": build_date_delta(exp.DateAdd, unit_mapping=DATE_DELTA_INTERVAL),
+            "DATEDIFF": _build_date_delta(exp.DateDiff, unit_mapping=DATE_DELTA_INTERVAL),
+            "DATENAME": _build_formatted_time(exp.TimeToStr, full_format_mapping=True),
+            "DATEPART": _build_formatted_time(exp.TimeToStr),
+            "DATETIMEFROMPARTS": _build_datetimefromparts,
+            "EOMONTH": _build_eomonth,
+            "FORMAT": _build_format,
             "GETDATE": exp.CurrentTimestamp.from_arg_list,
-            "HASHBYTES": _parse_hashbytes,
-            "IIF": exp.If.from_arg_list,
+            "HASHBYTES": _build_hashbytes,
             "ISNULL": exp.Coalesce.from_arg_list,
-            "JSON_VALUE": exp.JSONExtractScalar.from_arg_list,
-            "LEN": exp.Length.from_arg_list,
+            "JSON_QUERY": parser.build_extract_json_with_path(exp.JSONExtract),
+            "JSON_VALUE": parser.build_extract_json_with_path(exp.JSONExtractScalar),
+            "LEN": _build_with_arg_as_text(exp.Length),
+            "LEFT": _build_with_arg_as_text(exp.Left),
+            "RIGHT": _build_with_arg_as_text(exp.Right),
             "REPLICATE": exp.Repeat.from_arg_list,
             "SQUARE": lambda args: exp.Pow(this=seq_get(args, 0), expression=exp.Literal.number(2)),
             "SYSDATETIME": exp.CurrentTimestamp.from_arg_list,
             "SUSER_NAME": exp.CurrentUser.from_arg_list,
             "SUSER_SNAME": exp.CurrentUser.from_arg_list,
             "SYSTEM_USER": exp.CurrentUser.from_arg_list,
+            "TIMEFROMPARTS": _build_timefromparts,
         }
 
-        JOIN_HINTS = {
-            "LOOP",
-            "HASH",
-            "MERGE",
-            "REMOTE",
-        }
-
-        VAR_LENGTH_DATATYPES = {
-            DataType.Type.NVARCHAR,
-            DataType.Type.VARCHAR,
-            DataType.Type.CHAR,
-            DataType.Type.NCHAR,
-        }
+        JOIN_HINTS = {"LOOP", "HASH", "MERGE", "REMOTE"}
 
         RETURNS_TABLE_TOKENS = parser.Parser.ID_VAR_TOKENS - {
             TokenType.TABLE,
@@ -437,9 +527,21 @@ class TSQL(Dialect):
             TokenType.END: lambda self: self._parse_command(),
         }
 
-        LOG_DEFAULTS_TO_LN = True
+        def _parse_options(self) -> t.Optional[t.List[exp.Expression]]:
+            if not self._match(TokenType.OPTION):
+                return None
 
-        ALTER_TABLE_ADD_REQUIRED_FOR_EACH_COLUMN = False
+            def _parse_option() -> t.Optional[exp.Expression]:
+                option = self._parse_var_from_options(OPTIONS)
+                if not option:
+                    return None
+
+                self._match(TokenType.EQ)
+                return self.expression(
+                    exp.QueryOption, this=option, expression=self._parse_primary_or_var()
+                )
+
+            return self._parse_wrapped_csv(_parse_option)
 
         def _parse_projections(self) -> t.List[exp.Expression]:
             """
@@ -449,9 +551,11 @@ class TSQL(Dialect):
             See: https://learn.microsoft.com/en-us/sql/t-sql/queries/select-clause-transact-sql?view=sql-server-ver16#syntax
             """
             return [
-                exp.alias_(projection.expression, projection.this.this, copy=False)
-                if isinstance(projection, exp.EQ) and isinstance(projection.this, exp.Column)
-                else projection
+                (
+                    exp.alias_(projection.expression, projection.this.this, copy=False)
+                    if isinstance(projection, exp.EQ) and isinstance(projection.this, exp.Column)
+                    else projection
+                )
                 for projection in super()._parse_projections()
             ]
 
@@ -513,48 +617,13 @@ class TSQL(Dialect):
         def _parse_convert(
             self, strict: bool, safe: t.Optional[bool] = None
         ) -> t.Optional[exp.Expression]:
-            to = self._parse_types()
+            this = self._parse_types()
             self._match(TokenType.COMMA)
-            this = self._parse_conjunction()
-
-            if not to or not this:
-                return None
-
-            # Retrieve length of datatype and override to default if not specified
-            if seq_get(to.expressions, 0) is None and to.this in self.VAR_LENGTH_DATATYPES:
-                to = exp.DataType.build(to.this, expressions=[exp.Literal.number(30)], nested=False)
-
-            # Check whether a conversion with format is applicable
-            if self._match(TokenType.COMMA):
-                format_val = self._parse_number()
-                format_val_name = format_val.name if format_val else ""
-
-                if format_val_name not in TSQL.CONVERT_FORMAT_MAPPING:
-                    raise ValueError(
-                        f"CONVERT function at T-SQL does not support format style {format_val_name}"
-                    )
-
-                format_norm = exp.Literal.string(TSQL.CONVERT_FORMAT_MAPPING[format_val_name])
-
-                # Check whether the convert entails a string to date format
-                if to.this == DataType.Type.DATE:
-                    return self.expression(exp.StrToDate, this=this, format=format_norm)
-                # Check whether the convert entails a string to datetime format
-                elif to.this == DataType.Type.DATETIME:
-                    return self.expression(exp.StrToTime, this=this, format=format_norm)
-                # Check whether the convert entails a date to string format
-                elif to.this in self.VAR_LENGTH_DATATYPES:
-                    return self.expression(
-                        exp.Cast if strict else exp.TryCast,
-                        to=to,
-                        this=self.expression(exp.TimeToStr, this=this, format=format_norm),
-                        safe=safe,
-                    )
-                elif to.this == DataType.Type.TEXT:
-                    return self.expression(exp.TimeToStr, this=this, format=format_norm)
-
-            # Entails a simple cast without any format requirement
-            return self.expression(exp.Cast if strict else exp.TryCast, this=this, to=to, safe=safe)
+            args = [this, *self._parse_csv(self._parse_conjunction)]
+            convert = exp.Convert.from_arg_list(args)
+            convert.set("safe", safe)
+            convert.set("strict", strict)
+            return convert
 
         def _parse_user_defined_function(
             self, kind: t.Optional[TokenType] = None
@@ -620,6 +689,26 @@ class TSQL(Dialect):
 
             return self.expression(exp.UniqueColumnConstraint, this=this)
 
+        def _parse_partition(self) -> t.Optional[exp.Partition]:
+            if not self._match_text_seq("WITH", "(", "PARTITIONS"):
+                return None
+
+            def parse_range():
+                low = self._parse_bitwise()
+                high = self._parse_bitwise() if self._match_text_seq("TO") else None
+
+                return (
+                    self.expression(exp.PartitionRange, this=low, expression=high) if high else low
+                )
+
+            partition = self.expression(
+                exp.Partition, expressions=self._parse_wrapped_csv(parse_range)
+            )
+
+            self._match_r_paren()
+
+            return partition
+
     class Generator(generator.Generator):
         LIMIT_IS_TOP = True
         QUERY_HINTS = False
@@ -630,8 +719,11 @@ class TSQL(Dialect):
         COMPUTED_COLUMN_WITH_TYPE = False
         CTE_RECURSIVE_KEYWORD_REQUIRED = False
         ENSURE_BOOLS = True
-        NULL_ORDERING_SUPPORTED = False
+        NULL_ORDERING_SUPPORTED = None
         SUPPORTS_SINGLE_ARG_CONCAT = False
+        TABLESAMPLE_SEED_KEYWORD = "REPEATABLE"
+        SUPPORTS_SELECT_INTO = True
+        JSON_PATH_BRACKETED_KEY_SUPPORTED = False
 
         EXPRESSIONS_WITHOUT_NESTED_CTES = {
             exp.Delete,
@@ -643,6 +735,12 @@ class TSQL(Dialect):
             exp.Update,
         }
 
+        SUPPORTED_JSON_PATH_PARTS = {
+            exp.JSONPathKey,
+            exp.JSONPathRoot,
+            exp.JSONPathSubscript,
+        }
+
         TYPE_MAPPING = {
             **generator.Generator.TYPE_MAPPING,
             exp.DataType.Type.BOOLEAN: "BIT",
@@ -656,6 +754,9 @@ class TSQL(Dialect):
             exp.DataType.Type.VARIANT: "SQL_VARIANT",
         }
 
+        TYPE_MAPPING.pop(exp.DataType.Type.NCHAR)
+        TYPE_MAPPING.pop(exp.DataType.Type.NVARCHAR)
+
         TRANSFORMS = {
             **generator.Generator.TRANSFORMS,
             exp.AnyValue: any_value_to_max_sql,
@@ -669,11 +770,14 @@ class TSQL(Dialect):
             exp.GeneratedAsIdentityColumnConstraint: generatedasidentitycolumnconstraint_sql,
             exp.GroupConcat: _string_agg_sql,
             exp.If: rename_func("IIF"),
-            exp.Length: rename_func("LEN"),
+            exp.JSONExtract: _json_extract_sql,
+            exp.JSONExtractScalar: _json_extract_sql,
+            exp.LastDay: lambda self, e: self.func("EOMONTH", e.this),
             exp.Max: max_or_greatest,
             exp.MD5: lambda self, e: self.func("HASHBYTES", exp.Literal.string("MD5"), e.this),
             exp.Min: min_or_least,
             exp.NumberToStr: _format_sql,
+            exp.ParseJSON: lambda self, e: self.sql(e, "this"),
             exp.Select: transforms.preprocess(
                 [
                     transforms.eliminate_distinct_on,
@@ -681,6 +785,9 @@ class TSQL(Dialect):
                     transforms.eliminate_qualify,
                 ]
             ),
+            exp.StrPosition: lambda self, e: self.func(
+                "CHARINDEX", e.args.get("substr"), e.this, e.args.get("position")
+            ),
             exp.Subquery: transforms.preprocess([qualify_derived_table_outputs]),
             exp.SHA: lambda self, e: self.func("HASHBYTES", exp.Literal.string("SHA1"), e.this),
             exp.SHA2: lambda self, e: self.func(
@@ -689,9 +796,9 @@ class TSQL(Dialect):
             exp.TemporaryProperty: lambda self, e: "",
             exp.TimeStrToTime: timestrtotime_sql,
             exp.TimeToStr: _format_sql,
+            exp.Trim: trim_sql,
             exp.TsOrDsAdd: date_delta_sql("DATEADD", cast=True),
             exp.TsOrDsDiff: date_delta_sql("DATEDIFF"),
-            exp.TsOrDsToDate: ts_or_ds_to_date_sql("tsql"),
         }
 
         TRANSFORMS.pop(exp.ReturnsProperty)
@@ -701,6 +808,67 @@ class TSQL(Dialect):
             exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
         }
 
+        def convert_sql(self, expression: exp.Convert) -> str:
+            name = "TRY_CONVERT" if expression.args.get("safe") else "CONVERT"
+            return self.func(
+                name, expression.this, expression.expression, expression.args.get("style")
+            )
+
+        def queryoption_sql(self, expression: exp.QueryOption) -> str:
+            option = self.sql(expression, "this")
+            value = self.sql(expression, "expression")
+            if value:
+                optional_equal_sign = "= " if option in OPTIONS_THAT_REQUIRE_EQUAL else ""
+                return f"{option} {optional_equal_sign}{value}"
+            return option
+
+        def lateral_op(self, expression: exp.Lateral) -> str:
+            cross_apply = expression.args.get("cross_apply")
+            if cross_apply is True:
+                return "CROSS APPLY"
+            if cross_apply is False:
+                return "OUTER APPLY"
+
+            # TODO: perhaps we can check if the parent is a Join and transpile it appropriately
+            self.unsupported("LATERAL clause is not supported.")
+            return "LATERAL"
+
+        def timefromparts_sql(self, expression: exp.TimeFromParts) -> str:
+            nano = expression.args.get("nano")
+            if nano is not None:
+                nano.pop()
+                self.unsupported("Specifying nanoseconds is not supported in TIMEFROMPARTS.")
+
+            if expression.args.get("fractions") is None:
+                expression.set("fractions", exp.Literal.number(0))
+            if expression.args.get("precision") is None:
+                expression.set("precision", exp.Literal.number(0))
+
+            return rename_func("TIMEFROMPARTS")(self, expression)
+
+        def timestampfromparts_sql(self, expression: exp.TimestampFromParts) -> str:
+            zone = expression.args.get("zone")
+            if zone is not None:
+                zone.pop()
+                self.unsupported("Time zone is not supported in DATETIMEFROMPARTS.")
+
+            nano = expression.args.get("nano")
+            if nano is not None:
+                nano.pop()
+                self.unsupported("Specifying nanoseconds is not supported in DATETIMEFROMPARTS.")
+
+            if expression.args.get("milli") is None:
+                expression.set("milli", exp.Literal.number(0))
+
+            return rename_func("DATETIMEFROMPARTS")(self, expression)
+
+        def set_operation(self, expression: exp.Union, op: str) -> str:
+            limit = expression.args.get("limit")
+            if limit:
+                return self.sql(expression.limit(limit.pop(), copy=False))
+
+            return super().set_operation(expression, op)
+
         def setitem_sql(self, expression: exp.SetItem) -> str:
             this = expression.this
             if isinstance(this, exp.EQ) and not isinstance(this.left, exp.Parameter):
@@ -733,26 +901,34 @@ class TSQL(Dialect):
             return sql
 
         def create_sql(self, expression: exp.Create) -> str:
-            kind = self.sql(expression, "kind").upper()
+            kind = expression.kind
             exists = expression.args.pop("exists", None)
             sql = super().create_sql(expression)
 
+            like_property = expression.find(exp.LikeProperty)
+            if like_property:
+                ctas_expression = like_property.this
+            else:
+                ctas_expression = expression.expression
+
             table = expression.find(exp.Table)
 
             # Convert CTAS statement to SELECT .. INTO ..
-            if kind == "TABLE" and expression.expression:
-                ctas_with = expression.expression.args.get("with")
+            if kind == "TABLE" and ctas_expression:
+                ctas_with = ctas_expression.args.get("with")
                 if ctas_with:
                     ctas_with = ctas_with.pop()
 
-                subquery = expression.expression
-                if isinstance(subquery, exp.Subqueryable):
-                    subquery = subquery.subquery()
+                if isinstance(ctas_expression, exp.UNWRAPPED_QUERIES):
+                    ctas_expression = ctas_expression.subquery()
 
-                select_into = exp.select("*").from_(exp.alias_(subquery, "temp", table=True))
+                select_into = exp.select("*").from_(exp.alias_(ctas_expression, "temp", table=True))
                 select_into.set("into", exp.Into(this=table))
                 select_into.set("with", ctas_with)
 
+                if like_property:
+                    select_into.limit(0, copy=False)
+
                 sql = self.sql(select_into)
 
             if exists:
@@ -841,3 +1017,24 @@ class TSQL(Dialect):
             this = self.sql(expression, "this")
             expressions = self.expressions(expression, flat=True, sep=" ")
             return f"CONSTRAINT {this} {expressions}"
+
+        def length_sql(self, expression: exp.Length) -> str:
+            return self._uncast_text(expression, "LEN")
+
+        def right_sql(self, expression: exp.Right) -> str:
+            return self._uncast_text(expression, "RIGHT")
+
+        def left_sql(self, expression: exp.Left) -> str:
+            return self._uncast_text(expression, "LEFT")
+
+        def _uncast_text(self, expression: exp.Expression, name: str) -> str:
+            this = expression.this
+            if isinstance(this, exp.Cast) and this.is_type(exp.DataType.Type.TEXT):
+                this_sql = self.sql(this, "this")
+            else:
+                this_sql = self.sql(this)
+            expression_sql = self.sql(expression, "expression")
+            return self.func(name, this_sql, expression_sql if expression_sql else None)
+
+        def partition_sql(self, expression: exp.Partition) -> str:
+            return f"WITH (PARTITIONS({self.expressions(expression, flat=True)}))"
