@@ -1,5 +1,6 @@
 """
 .. include:: ../posts/sql_diff.md
+
 ----
 """
 
@@ -10,9 +11,8 @@ from collections import defaultdict
 from dataclasses import dataclass
 from heapq import heappop, heappush
 
-from sqlglot import Dialect
-from sqlglot import expressions as exp
-from sqlglot.helper import ensure_collection
+from sqlglot import Dialect, expressions as exp
+from sqlglot.helper import ensure_list
 
 
 @dataclass(frozen=True)
@@ -53,11 +53,17 @@ class Keep:
 
 
 if t.TYPE_CHECKING:
-    T = t.TypeVar("T")
+    from sqlglot._typing import T
+
     Edit = t.Union[Insert, Remove, Move, Update, Keep]
 
 
-def diff(source: exp.Expression, target: exp.Expression) -> t.List[Edit]:
+def diff(
+    source: exp.Expression,
+    target: exp.Expression,
+    matchings: t.List[t.Tuple[exp.Expression, exp.Expression]] | None = None,
+    **kwargs: t.Any,
+) -> t.List[Edit]:
     """
     Returns the list of changes between the source and the target expressions.
 
@@ -79,22 +85,52 @@ def diff(source: exp.Expression, target: exp.Expression) -> t.List[Edit]:
     Args:
         source: the source expression.
         target: the target expression against which the diff should be calculated.
+        matchings: the list of pre-matched node pairs which is used to help the algorithm's
+            heuristics produce better results for subtrees that are known by a caller to be matching.
+            Note: expression references in this list must refer to the same node objects that are
+            referenced in source / target trees.
 
     Returns:
         the list of Insert, Remove, Move, Update and Keep objects for each node in the source and the
         target expression trees. This list represents a sequence of steps needed to transform the source
         expression tree into the target one.
     """
-    return ChangeDistiller().diff(source.copy(), target.copy())
+    matchings = matchings or []
+    matching_ids = {id(n) for pair in matchings for n in pair}
+
+    def compute_node_mappings(
+        original: exp.Expression, copy: exp.Expression
+    ) -> t.Dict[int, exp.Expression]:
+        return {
+            id(old_node): new_node
+            for (old_node, _, _), (new_node, _, _) in zip(original.walk(), copy.walk())
+            if id(old_node) in matching_ids
+        }
+
+    source_copy = source.copy()
+    target_copy = target.copy()
+
+    node_mappings = {
+        **compute_node_mappings(source, source_copy),
+        **compute_node_mappings(target, target_copy),
+    }
+    matchings_copy = [(node_mappings[id(s)], node_mappings[id(t)]) for s, t in matchings]
+
+    return ChangeDistiller(**kwargs).diff(source_copy, target_copy, matchings=matchings_copy)
 
 
-LEAF_EXPRESSION_TYPES = (
+# The expression types for which Update edits are allowed.
+UPDATABLE_EXPRESSION_TYPES = (
     exp.Boolean,
     exp.DataType,
-    exp.Identifier,
     exp.Literal,
+    exp.Table,
+    exp.Column,
+    exp.Lambda,
 )
 
+IGNORED_LEAF_EXPRESSION_TYPES = (exp.Identifier,)
+
 
 class ChangeDistiller:
     """
@@ -108,16 +144,34 @@ class ChangeDistiller:
         self.t = t
         self._sql_generator = Dialect().generator()
 
-    def diff(self, source: exp.Expression, target: exp.Expression) -> t.List[Edit]:
+    def diff(
+        self,
+        source: exp.Expression,
+        target: exp.Expression,
+        matchings: t.List[t.Tuple[exp.Expression, exp.Expression]] | None = None,
+    ) -> t.List[Edit]:
+        matchings = matchings or []
+        pre_matched_nodes = {id(s): id(t) for s, t in matchings}
+        if len({n for pair in pre_matched_nodes.items() for n in pair}) != 2 * len(matchings):
+            raise ValueError("Each node can be referenced at most once in the list of matchings")
+
         self._source = source
         self._target = target
-        self._source_index = {id(n[0]): n[0] for n in source.bfs()}
-        self._target_index = {id(n[0]): n[0] for n in target.bfs()}
-        self._unmatched_source_nodes = set(self._source_index)
-        self._unmatched_target_nodes = set(self._target_index)
+        self._source_index = {
+            id(n): n
+            for n, *_ in self._source.bfs()
+            if not isinstance(n, IGNORED_LEAF_EXPRESSION_TYPES)
+        }
+        self._target_index = {
+            id(n): n
+            for n, *_ in self._target.bfs()
+            if not isinstance(n, IGNORED_LEAF_EXPRESSION_TYPES)
+        }
+        self._unmatched_source_nodes = set(self._source_index) - set(pre_matched_nodes)
+        self._unmatched_target_nodes = set(self._target_index) - set(pre_matched_nodes.values())
         self._bigram_histo_cache: t.Dict[int, t.DefaultDict[str, int]] = {}
 
-        matching_set = self._compute_matching_set()
+        matching_set = self._compute_matching_set() | {(s, t) for s, t in pre_matched_nodes.items()}
         return self._generate_edit_script(matching_set)
 
     def _generate_edit_script(self, matching_set: t.Set[t.Tuple[int, int]]) -> t.List[Edit]:
@@ -129,7 +183,10 @@ class ChangeDistiller:
         for kept_source_node_id, kept_target_node_id in matching_set:
             source_node = self._source_index[kept_source_node_id]
             target_node = self._target_index[kept_target_node_id]
-            if not isinstance(source_node, LEAF_EXPRESSION_TYPES) or source_node == target_node:
+            if (
+                not isinstance(source_node, UPDATABLE_EXPRESSION_TYPES)
+                or source_node == target_node
+            ):
                 edit_script.extend(
                     self._generate_move_edits(source_node, target_node, matching_set)
                 )
@@ -159,10 +216,10 @@ class ChangeDistiller:
         matching_set = leaves_matching_set.copy()
 
         ordered_unmatched_source_nodes = {
-            id(n[0]): None for n in self._source.bfs() if id(n[0]) in self._unmatched_source_nodes
+            id(n): None for n, *_ in self._source.bfs() if id(n) in self._unmatched_source_nodes
         }
         ordered_unmatched_target_nodes = {
-            id(n[0]): None for n in self._target.bfs() if id(n[0]) in self._unmatched_target_nodes
+            id(n): None for n, *_ in self._target.bfs() if id(n) in self._unmatched_target_nodes
         }
 
         for source_node_id in ordered_unmatched_source_nodes:
@@ -200,7 +257,7 @@ class ChangeDistiller:
         return matching_set
 
     def _compute_leaf_matching_set(self) -> t.Set[t.Tuple[int, int]]:
-        candidate_matchings: t.List[t.Tuple[float, int, exp.Expression, exp.Expression]] = []
+        candidate_matchings: t.List[t.Tuple[float, int, int, exp.Expression, exp.Expression]] = []
         source_leaves = list(_get_leaves(self._source))
         target_leaves = list(_get_leaves(self._target))
         for source_leaf in source_leaves:
@@ -212,6 +269,7 @@ class ChangeDistiller:
                             candidate_matchings,
                             (
                                 -similarity_score,
+                                -_parent_similarity_score(source_leaf, target_leaf),
                                 len(candidate_matchings),
                                 source_leaf,
                                 target_leaf,
@@ -221,7 +279,7 @@ class ChangeDistiller:
         # Pick best matchings based on the highest score
         matching_set = set()
         while candidate_matchings:
-            _, _, source_leaf, target_leaf = heappop(candidate_matchings)
+            _, _, _, source_leaf, target_leaf = heappop(candidate_matchings)
             if (
                 id(source_leaf) in self._unmatched_source_nodes
                 and id(target_leaf) in self._unmatched_target_nodes
@@ -264,11 +322,10 @@ class ChangeDistiller:
 def _get_leaves(expression: exp.Expression) -> t.Iterator[exp.Expression]:
     has_child_exprs = False
 
-    for a in expression.args.values():
-        for node in ensure_collection(a):
-            if isinstance(node, exp.Expression):
-                has_child_exprs = True
-                yield from _get_leaves(node)
+    for _, node in expression.iter_expressions():
+        if not isinstance(node, IGNORED_LEAF_EXPRESSION_TYPES):
+            has_child_exprs = True
+            yield from _get_leaves(node)
 
     if not has_child_exprs:
         yield expression
@@ -287,12 +344,25 @@ def _is_same_type(source: exp.Expression, target: exp.Expression) -> bool:
     return False
 
 
+def _parent_similarity_score(
+    source: t.Optional[exp.Expression], target: t.Optional[exp.Expression]
+) -> int:
+    if source is None or target is None or type(source) is not type(target):
+        return 0
+
+    return 1 + _parent_similarity_score(source.parent, target.parent)
+
+
 def _expression_only_args(expression: exp.Expression) -> t.List[exp.Expression]:
     args: t.List[t.Union[exp.Expression, t.List]] = []
     if expression:
         for a in expression.args.values():
-            args.extend(ensure_collection(a))
-    return [a for a in args if isinstance(a, exp.Expression)]
+            args.extend(ensure_list(a))
+    return [
+        a
+        for a in args
+        if isinstance(a, exp.Expression) and not isinstance(a, IGNORED_LEAF_EXPRESSION_TYPES)
+    ]
 
 
 def _lcs(
