@@ -6,7 +6,7 @@ from functools import wraps
 
 from sqlglot import exp
 from sqlglot.generator import Generator
-from sqlglot.helper import PYTHON_VERSION
+from sqlglot.helper import PYTHON_VERSION, is_int, seq_get
 
 
 class reverse_key:
@@ -100,9 +100,19 @@ def substring(this, start=None, length=None):
 @null_if_any
 def cast(this, to):
     if to == exp.DataType.Type.DATE:
-        return datetime.date.fromisoformat(this)
-    if to == exp.DataType.Type.DATETIME:
-        return datetime.datetime.fromisoformat(this)
+        if isinstance(this, datetime.datetime):
+            return this.date()
+        if isinstance(this, datetime.date):
+            return this
+        if isinstance(this, str):
+            return datetime.date.fromisoformat(this)
+    if to in (exp.DataType.Type.DATETIME, exp.DataType.Type.TIMESTAMP):
+        if isinstance(this, datetime.datetime):
+            return this
+        if isinstance(this, datetime.date):
+            return datetime.datetime(this.year, this.month, this.day)
+        if isinstance(this, str):
+            return datetime.datetime.fromisoformat(this)
     if to == exp.DataType.Type.BOOLEAN:
         return bool(this)
     if to in exp.DataType.TEXT_TYPES:
@@ -111,7 +121,7 @@ def cast(this, to):
         return float(this)
     if to in exp.DataType.NUMERIC_TYPES:
         return int(this)
-    raise NotImplementedError(f"Casting to '{to}' not implemented.")
+    raise NotImplementedError(f"Casting {this} to '{to}' not implemented.")
 
 
 def ordered(this, desc, nulls_first):
@@ -122,17 +132,38 @@ def ordered(this, desc, nulls_first):
 
 @null_if_any
 def interval(this, unit):
-    unit = unit.lower()
-    plural = unit + "s"
+    plural = unit + "S"
     if plural in Generator.TIME_PART_SINGULARS:
         unit = plural
-    return datetime.timedelta(**{unit: float(this)})
+    return datetime.timedelta(**{unit.lower(): float(this)})
+
+
+@null_if_any("this", "expression")
+def arrayjoin(this, expression, null=None):
+    return expression.join(x for x in (x if x is not None else null for x in this) if x is not None)
+
+
+@null_if_any("this", "expression")
+def jsonextract(this, expression):
+    for path_segment in expression:
+        if isinstance(this, dict):
+            this = this.get(path_segment)
+        elif isinstance(this, list) and is_int(path_segment):
+            this = seq_get(this, int(path_segment))
+        else:
+            raise NotImplementedError(f"Unable to extract value for {this} at {path_segment}.")
+
+        if this is None:
+            break
+
+    return this
 
 
 ENV = {
     "exp": exp,
     # aggs
     "ARRAYAGG": list,
+    "ARRAYUNIQUEAGG": filter_nulls(lambda acc: list(set(acc))),
     "AVG": filter_nulls(statistics.fmean if PYTHON_VERSION >= (3, 8) else statistics.mean),  # type: ignore
     "COUNT": filter_nulls(lambda acc: sum(1 for _ in acc), False),
     "MAX": filter_nulls(max),
@@ -142,6 +173,7 @@ ENV = {
     "ABS": null_if_any(lambda this: abs(this)),
     "ADD": null_if_any(lambda e, this: e + this),
     "ARRAYANY": null_if_any(lambda arr, func: any(func(e) for e in arr)),
+    "ARRAYJOIN": arrayjoin,
     "BETWEEN": null_if_any(lambda this, low, high: low <= this and this <= high),
     "BITWISEAND": null_if_any(lambda this, e: this & e),
     "BITWISELEFTSHIFT": null_if_any(lambda this, e: this << e),
@@ -153,6 +185,7 @@ ENV = {
     "CONCAT": null_if_any(lambda *args: "".join(args)),
     "SAFECONCAT": null_if_any(lambda *args: "".join(str(arg) for arg in args)),
     "CONCATWS": null_if_any(lambda this, *args: this.join(args)),
+    "DATEDIFF": null_if_any(lambda this, expression, *_: (this - expression).days),
     "DATESTRTODATE": null_if_any(lambda arg: datetime.date.fromisoformat(arg)),
     "DIV": null_if_any(lambda e, this: e / this),
     "DOT": null_if_any(lambda e, this: e[this]),
@@ -163,6 +196,7 @@ ENV = {
     "IF": lambda predicate, true, false: true if predicate else false,
     "INTDIV": null_if_any(lambda e, this: e // this),
     "INTERVAL": interval,
+    "JSONEXTRACT": jsonextract,
     "LEFT": null_if_any(lambda this, e: this[:e]),
     "LIKE": null_if_any(
         lambda this, e: bool(re.match(e.replace("_", ".").replace("%", ".*"), this))
@@ -191,4 +225,10 @@ ENV = {
     "CURRENTTIME": datetime.datetime.now,
     "CURRENTDATE": datetime.date.today,
     "STRFTIME": null_if_any(lambda fmt, arg: datetime.datetime.fromisoformat(arg).strftime(fmt)),
+    "TRIM": null_if_any(lambda this, e=None: this.strip(e)),
+    "STRUCT": lambda *args: {
+        args[x]: args[x + 1]
+        for x in range(0, len(args), 2)
+        if (args[x + 1] is not None and args[x] is not None)
+    },
 }
