@@ -1,11 +1,12 @@
 from __future__ import annotations
 
+import datetime
 import inspect
 import logging
 import re
 import sys
 import typing as t
-from collections.abc import Collection
+from collections.abc import Collection, Set
 from contextlib import contextmanager
 from copy import copy
 from enum import Enum
@@ -52,13 +53,11 @@ def seq_get(seq: t.Sequence[T], index: int) -> t.Optional[T]:
 
 
 @t.overload
-def ensure_list(value: t.Collection[T]) -> t.List[T]:
-    ...
+def ensure_list(value: t.Collection[T]) -> t.List[T]: ...
 
 
 @t.overload
-def ensure_list(value: T) -> t.List[T]:
-    ...
+def ensure_list(value: T) -> t.List[T]: ...
 
 
 def ensure_list(value):
@@ -80,13 +79,11 @@ def ensure_list(value):
 
 
 @t.overload
-def ensure_collection(value: t.Collection[T]) -> t.Collection[T]:
-    ...
+def ensure_collection(value: t.Collection[T]) -> t.Collection[T]: ...
 
 
 @t.overload
-def ensure_collection(value: T) -> t.Collection[T]:
-    ...
+def ensure_collection(value: T) -> t.Collection[T]: ...
 
 
 def ensure_collection(value):
@@ -235,7 +232,7 @@ def tsort(dag: t.Dict[T, t.Set[T]]) -> t.List[T]:
 
     for node, deps in tuple(dag.items()):
         for dep in deps:
-            if not dep in dag:
+            if dep not in dag:
                 dag[dep] = set()
 
     while dag:
@@ -283,7 +280,7 @@ def csv_reader(read_csv: exp.ReadCSV) -> t.Any:
     file = open_file(read_csv.name)
 
     delimiter = ","
-    args = iter(arg.name for arg in args)
+    args = iter(arg.name for arg in args)  # type: ignore
     for k, v in zip(args, args):
         if k == "delimiter":
             delimiter = v
@@ -319,6 +316,14 @@ def find_new_name(taken: t.Collection[str], base: str) -> str:
     return new
 
 
+def is_int(text: str) -> bool:
+    try:
+        int(text)
+        return True
+    except ValueError:
+        return False
+
+
 def name_sequence(prefix: str) -> t.Callable[[], str]:
     """Returns a name generator given a prefix (e.g. a0, a1, a2, ... if the prefix is "a")."""
     sequence = count()
@@ -463,3 +468,55 @@ def merge_ranges(ranges: t.List[t.Tuple[A, A]]) -> t.List[t.Tuple[A, A]]:
             merged.append((start, end))
 
     return merged
+
+
+def is_iso_date(text: str) -> bool:
+    try:
+        datetime.date.fromisoformat(text)
+        return True
+    except ValueError:
+        return False
+
+
+def is_iso_datetime(text: str) -> bool:
+    try:
+        datetime.datetime.fromisoformat(text)
+        return True
+    except ValueError:
+        return False
+
+
+# Interval units that operate on date components
+DATE_UNITS = {"day", "week", "month", "quarter", "year", "year_month"}
+
+
+def is_date_unit(expression: t.Optional[exp.Expression]) -> bool:
+    return expression is not None and expression.name.lower() in DATE_UNITS
+
+
+K = t.TypeVar("K")
+V = t.TypeVar("V")
+
+
+class SingleValuedMapping(t.Mapping[K, V]):
+    """
+    Mapping where all keys return the same value.
+
+    This rigamarole is meant to avoid copying keys, which was originally intended
+    as an optimization while qualifying columns for tables with lots of columns.
+    """
+
+    def __init__(self, keys: t.Collection[K], value: V):
+        self._keys = keys if isinstance(keys, Set) else set(keys)
+        self._value = value
+
+    def __getitem__(self, key: K) -> V:
+        if key in self._keys:
+            return self._value
+        raise KeyError(key)
+
+    def __len__(self) -> int:
+        return len(self._keys)
+
+    def __iter__(self) -> t.Iterator[K]:
+        return iter(self._keys)
