@@ -12,7 +12,12 @@ def isolate_table_selects(expression, schema=None):
             continue
 
         for _, source in scope.selected_sources.values():
-            if not isinstance(source, exp.Table) or not schema.column_names(source):
+            if (
+                not isinstance(source, exp.Table)
+                or not schema.column_names(source)
+                or isinstance(source.parent, exp.Subquery)
+                or isinstance(source.parent.parent, exp.Table)
+            ):
                 continue
 
             if not source.alias:
