@@ -1,18 +1,24 @@
+from __future__ import annotations
+
 import itertools
 import typing as t
 
 from sqlglot import alias, exp
-from sqlglot._typing import E
+from sqlglot.dialects.dialect import DialectType
 from sqlglot.helper import csv_reader, name_sequence
 from sqlglot.optimizer.scope import Scope, traverse_scope
 from sqlglot.schema import Schema
 
+if t.TYPE_CHECKING:
+    from sqlglot._typing import E
+
 
 def qualify_tables(
     expression: E,
-    db: t.Optional[str] = None,
-    catalog: t.Optional[str] = None,
+    db: t.Optional[str | exp.Identifier] = None,
+    catalog: t.Optional[str | exp.Identifier] = None,
     schema: t.Optional[Schema] = None,
+    dialect: DialectType = None,
 ) -> E:
     """
     Rewrite sqlglot AST to have fully qualified tables. Join constructs such as
@@ -33,11 +39,26 @@ def qualify_tables(
         db: Database name
         catalog: Catalog name
         schema: A schema to populate
+        dialect: The dialect to parse catalog and schema into.
 
     Returns:
         The qualified expression.
     """
     next_alias_name = name_sequence("_q_")
+    db = exp.parse_identifier(db, dialect=dialect) if db else None
+    catalog = exp.parse_identifier(catalog, dialect=dialect) if catalog else None
+
+    def _qualify(table: exp.Table) -> None:
+        if isinstance(table.this, exp.Identifier):
+            if not table.args.get("db"):
+                table.set("db", db)
+            if not table.args.get("catalog") and table.args.get("db"):
+                table.set("catalog", catalog)
+
+    if not isinstance(expression, exp.Query):
+        for node, *_ in expression.walk(prune=lambda n, *_: isinstance(n, exp.Query)):
+            if isinstance(node, exp.Table):
+                _qualify(node)
 
     for scope in traverse_scope(expression):
         for derived_table in itertools.chain(scope.ctes, scope.derived_tables):
@@ -59,17 +80,17 @@ def qualify_tables(
 
         for name, source in scope.sources.items():
             if isinstance(source, exp.Table):
-                if isinstance(source.this, exp.Identifier):
-                    if not source.args.get("db"):
-                        source.set("db", exp.to_identifier(db))
-                    if not source.args.get("catalog") and source.args.get("db"):
-                        source.set("catalog", exp.to_identifier(catalog))
+                _qualify(source)
 
+                pivots = pivots = source.args.get("pivots")
                 if not source.alias:
+                    # Don't add the pivot's alias to the pivoted table, use the table's name instead
+                    if pivots and pivots[0].alias == name:
+                        name = source.name
+
                     # Mutates the source by attaching an alias to it
                     alias(source, name or source.name or next_alias_name(), copy=False, table=True)
 
-                pivots = source.args.get("pivots")
                 if pivots and not pivots[0].alias:
                     pivots[0].set(
                         "alias", exp.TableAlias(this=exp.to_identifier(next_alias_name()))
@@ -96,5 +117,14 @@ def qualify_tables(
                 if isinstance(udtf, exp.Values) and not table_alias.columns:
                     for i, e in enumerate(udtf.expressions[0].expressions):
                         table_alias.append("columns", exp.to_identifier(f"_col_{i}"))
+            else:
+                for node, parent, _ in scope.walk():
+                    if (
+                        isinstance(node, exp.Table)
+                        and not node.alias
+                        and isinstance(parent, (exp.From, exp.Join))
+                    ):
+                        # Mutates the table by attaching an alias to it
+                        alias(node, node.name, copy=False, table=True)
 
     return expression
