@@ -41,7 +41,7 @@ def simplify(
 
     Args:
         expression (sqlglot.Expression): expression to simplify
-        constant_propagation: whether or not the constant propagation rule should be used
+        constant_propagation: whether the constant propagation rule should be used
 
     Returns:
         sqlglot.Expression: simplified expression
@@ -100,10 +100,10 @@ def simplify(
         node = simplify_parens(node)
         node = simplify_datetrunc(node, dialect)
         node = sort_comparison(node)
+        node = simplify_startswith(node)
 
         if root:
             expression.replace(node)
-
         return node
 
     expression = while_changing(expression, _simplify)
@@ -173,16 +173,20 @@ def simplify_not(expression):
         if isinstance(this, exp.Paren):
             condition = this.unnest()
             if isinstance(condition, exp.And):
-                return exp.or_(
-                    exp.not_(condition.left, copy=False),
-                    exp.not_(condition.right, copy=False),
-                    copy=False,
+                return exp.paren(
+                    exp.or_(
+                        exp.not_(condition.left, copy=False),
+                        exp.not_(condition.right, copy=False),
+                        copy=False,
+                    )
                 )
             if isinstance(condition, exp.Or):
-                return exp.and_(
-                    exp.not_(condition.left, copy=False),
-                    exp.not_(condition.right, copy=False),
-                    copy=False,
+                return exp.paren(
+                    exp.and_(
+                        exp.not_(condition.left, copy=False),
+                        exp.not_(condition.right, copy=False),
+                        copy=False,
+                    )
                 )
             if is_null(condition):
                 return exp.null()
@@ -489,7 +493,7 @@ def simplify_equality(expression: exp.Expression) -> exp.Expression:
     if isinstance(expression, COMPARISONS):
         l, r = expression.left, expression.right
 
-        if not l.__class__ in INVERSE_OPS:
+        if l.__class__ not in INVERSE_OPS:
             return expression
 
         if r.is_number:
@@ -713,8 +717,7 @@ def simplify_concat(expression):
     """Reduces all groups that contain string literals by concatenating them."""
     if not isinstance(expression, CONCATS) or (
         # We can't reduce a CONCAT_WS call if we don't statically know the separator
-        isinstance(expression, exp.ConcatWs)
-        and not expression.expressions[0].is_string
+        isinstance(expression, exp.ConcatWs) and not expression.expressions[0].is_string
     ):
         return expression
 
@@ -776,6 +779,26 @@ def simplify_conditionals(expression):
     return expression
 
 
+def simplify_startswith(expression: exp.Expression) -> exp.Expression:
+    """
+    Reduces a prefix check to either TRUE or FALSE if both the string and the
+    prefix are statically known.
+
+    Example:
+        >>> from sqlglot import parse_one
+        >>> simplify_startswith(parse_one("STARTSWITH('foo', 'f')")).sql()
+        'TRUE'
+    """
+    if (
+        isinstance(expression, exp.StartsWith)
+        and expression.this.is_string
+        and expression.expression.is_string
+    ):
+        return exp.convert(expression.name.startswith(expression.expression.name))
+
+    return expression
+
+
 DateRange = t.Tuple[datetime.date, datetime.date]
 
 
@@ -1047,9 +1070,11 @@ def extract_interval(expression):
 def date_literal(date):
     return exp.cast(
         exp.Literal.string(date),
-        exp.DataType.Type.DATETIME
-        if isinstance(date, datetime.datetime)
-        else exp.DataType.Type.DATE,
+        (
+            exp.DataType.Type.DATETIME
+            if isinstance(date, datetime.datetime)
+            else exp.DataType.Type.DATE
+        ),
     )
 
 
@@ -1160,7 +1185,7 @@ def gen(expression: t.Any) -> str:
 GEN_MAP = {
     exp.Add: lambda e: _binary(e, "+"),
     exp.And: lambda e: _binary(e, "AND"),
-    exp.Anonymous: lambda e: f"{e.this} {','.join(gen(e) for e in e.expressions)}",
+    exp.Anonymous: lambda e: _anonymous(e),
     exp.Between: lambda e: f"{gen(e.this)} BETWEEN {gen(e.args.get('low'))} AND {gen(e.args.get('high'))}",
     exp.Boolean: lambda e: "TRUE" if e.this else "FALSE",
     exp.Bracket: lambda e: f"{gen(e.this)}[{gen(e.expressions)}]",
@@ -1194,6 +1219,20 @@ GEN_MAP = {
 }
 
 
+def _anonymous(e: exp.Anonymous) -> str:
+    this = e.this
+    if isinstance(this, str):
+        name = this.upper()
+    elif isinstance(this, exp.Identifier):
+        name = f'"{this.name}"' if this.quoted else this.name.upper()
+    else:
+        raise ValueError(
+            f"Anonymous.this expects a str or an Identifier, got '{this.__class__.__name__}'."
+        )
+
+    return f"{name} {','.join(gen(e) for e in e.expressions)}"
+
+
 def _binary(e: exp.Binary, op: str) -> str:
     return f"{gen(e.left)} {op} {gen(e.right)}"
 
