@@ -178,15 +178,6 @@ impl<'a> TokenizerState<'a> {
         Ok(())
     }
 
-    fn peek(&self, i: usize) -> Result<char, TokenizerError> {
-        let index = self.current + i;
-        if index < self.size {
-            self.char_at(index)
-        } else {
-            Ok('\0')
-        }
-    }
-
     fn chars(&self, size: usize) -> String {
         let start = self.current - 1;
         let end = start + size;
@@ -399,6 +390,19 @@ impl<'a> TokenizerState<'a> {
             } else if *token_type == self.token_types.bit_string {
                 (Some(2), *token_type, end.clone())
             } else if *token_type == self.token_types.heredoc_string {
+                if self.settings.heredoc_tag_is_identifier
+                    && !self.is_identifier(self.peek_char)
+                    && self.peek_char.to_string() != *end
+                {
+                    if self.token_types.heredoc_string_alternative != self.token_types.var {
+                        self.add(self.token_types.heredoc_string_alternative, None)?
+                    } else {
+                        self.scan_var()?
+                    };
+
+                    return Ok(true)
+                };
+
                 self.advance(1)?;
                 let tag = if self.current_char.to_string() == *end {
                     String::from("")
@@ -456,20 +460,15 @@ impl<'a> TokenizerState<'a> {
             if self.peek_char.is_digit(10) {
                 self.advance(1)?;
             } else if self.peek_char == '.' && !decimal {
-                let after = self.peek(1)?;
-                if after.is_digit(10) || !after.is_alphabetic() {
-                    decimal = true;
-                    self.advance(1)?;
-                } else {
-                    return self.add(self.token_types.var, None);
-                }
+                decimal = true;
+                self.advance(1)?;
             } else if (self.peek_char == '-' || self.peek_char == '+') && scientific == 1 {
                 scientific += 1;
                 self.advance(1)?;
             } else if self.peek_char.to_ascii_uppercase() == 'E' && scientific == 0 {
                 scientific += 1;
                 self.advance(1)?;
-            } else if self.peek_char.is_alphabetic() || self.peek_char == '_' {
+            } else if self.is_identifier(self.peek_char) {
                 let number_text = self.text();
                 let mut literal = String::from("");
 
@@ -643,6 +642,10 @@ impl<'a> TokenizerState<'a> {
         Ok(text)
     }
 
+    fn is_identifier(&mut self, name: char) -> bool {
+        name.is_alphabetic() || name == '_'
+    }
+
     fn extract_value(&mut self) -> Result<String, TokenizerError> {
         loop {
             if !self.peek_char.is_whitespace()
