@@ -4,11 +4,17 @@ from tests.dialects.test_dialect import Validator
 class TestDrill(Validator):
     dialect = "drill"
 
+    def test_drill(self):
+        self.validate_all(
+            "DATE_FORMAT(a, 'yyyy')",
+            write={"drill": "TO_CHAR(a, 'yyyy')"},
+        )
+
     def test_string_literals(self):
         self.validate_all(
             "SELECT '2021-01-01' + INTERVAL 1 MONTH",
             write={
-                "mysql": "SELECT '2021-01-01' + INTERVAL 1 MONTH",
+                "mysql": "SELECT '2021-01-01' + INTERVAL '1' MONTH",
             },
         )
 
@@ -34,11 +40,11 @@ class TestDrill(Validator):
         self.validate_all(
             "'\\\\a'",
             read={
-                "presto": "'\\a'",
+                "presto": "'\\\\a'",
             },
             write={
-                "duckdb": "'\\a'",
-                "presto": "'\\a'",
+                "duckdb": "'\\\\a'",
+                "presto": "'\\\\a'",
                 "hive": "'\\\\a'",
                 "spark": "'\\\\a'",
             },
@@ -51,3 +57,16 @@ class TestDrill(Validator):
                 "drill": "SELECT * FROM table(dfs.`test_data.xlsx`(type => 'excel', sheetName => 'secondSheet'))",
             },
         )
+
+    def test_validate_pivot(self):
+        self.validate_all(
+            "SELECT * FROM (SELECT education_level, salary, marital_status, "
+            "EXTRACT(year FROM age(birth_date)) age FROM cp.`employee.json`) PIVOT (avg(salary) AS "
+            "avg_salary, avg(age) AS avg_age FOR marital_status IN ('M' married, 'S' single))",
+            write={
+                "drill": "SELECT * FROM (SELECT education_level, salary, marital_status, "
+                "EXTRACT(year FROM age(birth_date)) AS age FROM cp.`employee.json`) "
+                "PIVOT(avg(salary) AS avg_salary, avg(age) AS avg_age FOR marital_status "
+                "IN ('M' AS married, 'S' AS single))"
+            },
+        )
