@@ -42,11 +42,13 @@ class TestTeradata(Validator):
         )
 
     def test_statistics(self):
-        self.validate_identity("COLLECT STATISTICS ON tbl INDEX(col)")
-        self.validate_identity("COLLECT STATS ON tbl COLUMNS(col)")
-        self.validate_identity("COLLECT STATS COLUMNS(col) ON tbl")
-        self.validate_identity("HELP STATISTICS personel.employee")
-        self.validate_identity("HELP STATISTICS personnel.employee FROM my_qcd")
+        self.validate_identity("COLLECT STATISTICS ON tbl INDEX(col)", check_command_warning=True)
+        self.validate_identity("COLLECT STATS ON tbl COLUMNS(col)", check_command_warning=True)
+        self.validate_identity("COLLECT STATS COLUMNS(col) ON tbl", check_command_warning=True)
+        self.validate_identity("HELP STATISTICS personel.employee", check_command_warning=True)
+        self.validate_identity(
+            "HELP STATISTICS personnel.employee FROM my_qcd", check_command_warning=True
+        )
 
     def test_create(self):
         self.validate_identity(
@@ -147,6 +149,9 @@ class TestTeradata(Validator):
     def test_mod(self):
         self.validate_all("a MOD b", write={"teradata": "a MOD b", "mysql": "a % b"})
 
+    def test_power(self):
+        self.validate_all("a ** b", write={"teradata": "a ** b", "mysql": "POWER(a, b)"})
+
     def test_abbrev(self):
         self.validate_identity("a LT b", "a < b")
         self.validate_identity("a LE b", "a <= b")
@@ -191,3 +196,14 @@ class TestTeradata(Validator):
             },
         )
         self.validate_identity("CAST('1992-01' AS FORMAT 'YYYY-DD')")
+
+        self.validate_all(
+            "TRYCAST('-2.5' AS DECIMAL(5, 2))",
+            read={
+                "snowflake": "TRY_CAST('-2.5' AS DECIMAL(5, 2))",
+            },
+            write={
+                "snowflake": "TRY_CAST('-2.5' AS DECIMAL(5, 2))",
+                "teradata": "TRYCAST('-2.5' AS DECIMAL(5, 2))",
+            },
+        )
