@@ -79,7 +79,7 @@ WITH "cte" AS (
 )
 SELECT
   "cte"."a" AS "a"
-FROM "cte";
+FROM "cte" AS "cte";
 
 # title: Chained CTEs
 WITH cte1 AS (
@@ -103,11 +103,11 @@ WITH "cte1" AS (
 )
 SELECT
   "cte1"."a" AS "a"
-FROM "cte1"
+FROM "cte1" AS "cte1"
 UNION ALL
 SELECT
   "cte1"."a" + 1 AS "a"
-FROM "cte1";
+FROM "cte1" AS "cte1";
 
 # title: Correlated subquery
 SELECT a, SUM(b) AS sum_b
@@ -223,14 +223,14 @@ WITH "m" AS (
   SELECT
     "m"."a" AS "a",
     "m"."b" AS "b"
-  FROM "m"
+  FROM "m" AS "m"
   WHERE
     "m"."a" = 1
 ), "o" AS (
   SELECT
     "m"."a" AS "a",
     "m"."b" AS "b"
-  FROM "m"
+  FROM "m" AS "m"
   WHERE
     "m"."a" = 2
 )
@@ -238,10 +238,10 @@ SELECT
   "n"."a" AS "a",
   "n"."b" AS "b",
   "o"."b" AS "b"
-FROM "n"
+FROM "n" AS "n"
 JOIN "n" AS "n2"
   ON "n"."a" = "n2"."a"
-FULL JOIN "o"
+FULL JOIN "o" AS "o"
   ON "n"."a" = "o"."a"
 WHERE
   "o"."b" > 0;
@@ -355,7 +355,7 @@ WITH RECURSIVE "base" AS (
     "cte1"."a" AS "a",
     "cte1"."b" AS "b",
     "cte1"."c" AS "c"
-  FROM "cte1"
+  FROM "cte1" AS "cte1"
   WHERE
     "cte1"."a" < 1
 )
@@ -363,7 +363,7 @@ SELECT
   "cte1"."a" AS "a",
   "cte1"."b" AS "b",
   "cte1"."c" AS "c"
-FROM "cte1";
+FROM "cte1" AS "cte1";
 
 # title: right join should not push down to from
 SELECT x.a, y.b
@@ -626,8 +626,7 @@ SELECT
   "_q_0"."G" AS "G",
   "_q_0"."'x'" AS "'x'",
   "_q_0"."'y'" AS "'y'"
-FROM "U" AS "U" PIVOT(SUM("U"."F") FOR "U"."H" IN ('x', 'y')) AS "_q_0"
-;
+FROM "U" AS "U" PIVOT(SUM("U"."F") FOR "U"."H" IN ('x', 'y')) AS "_q_0";
 
 # title: selecting all columns from a pivoted source and generating spark
 # note: spark doesn't allow pivot aliases or qualified columns for the pivot's "field" (`h`)
@@ -644,7 +643,111 @@ FROM (
   FROM `u` AS `u` PIVOT(SUM(`u`.`f`) FOR `h` IN ('x', 'y'))
 ) AS `_q_0`;
 
-# title: quoting is maintained
+# title: selecting all columns from a pivoted source, pivot has column aliases
+# execute: false
+# dialect: snowflake
+WITH source AS (
+  SELECT
+    id,
+    key,
+    value,
+    timestamp_1,
+    timestamp_2
+  FROM DB_NAME.SCHEMA_NAME.TABLE_NAME
+),
+enriched AS (
+  SELECT * FROM source
+  PIVOT(MAX(value) FOR key IN ('a', 'b', 'c'))
+  AS final (id, timestamp_1, timestamp_2, col_1, col_2, col_3)
+)
+SELECT id, timestamp_1 FROM enriched;
+WITH "SOURCE" AS (
+  SELECT
+    "TABLE_NAME"."ID" AS "ID",
+    "TABLE_NAME"."KEY" AS "KEY",
+    "TABLE_NAME"."VALUE" AS "VALUE",
+    "TABLE_NAME"."TIMESTAMP_1" AS "TIMESTAMP_1",
+    "TABLE_NAME"."TIMESTAMP_2" AS "TIMESTAMP_2"
+  FROM "DB_NAME"."SCHEMA_NAME"."TABLE_NAME" AS "TABLE_NAME"
+)
+SELECT
+  "FINAL"."ID" AS "ID",
+  "FINAL"."TIMESTAMP_1" AS "TIMESTAMP_1"
+FROM "SOURCE" AS "SOURCE" PIVOT(MAX("SOURCE"."VALUE") FOR "SOURCE"."KEY" IN ('a', 'b', 'c')) AS "FINAL"("ID", "TIMESTAMP_1", "TIMESTAMP_2", "COL_1", "COL_2", "COL_3");
+
+# title: unpivoted table source with a single value column, unpivot columns can't be qualified
+# execute: false
+# dialect: snowflake
+SELECT * FROM m_sales AS m_sales(empid, dept, jan, feb) UNPIVOT(sales FOR month IN (jan, feb)) ORDER BY empid;
+SELECT
+  "_q_0"."EMPID" AS "EMPID",
+  "_q_0"."DEPT" AS "DEPT",
+  "_q_0"."MONTH" AS "MONTH",
+  "_q_0"."SALES" AS "SALES"
+FROM "M_SALES" AS "M_SALES"("EMPID", "DEPT", "JAN", "FEB") UNPIVOT("SALES" FOR "MONTH" IN ("JAN", "FEB")) AS "_q_0"
+ORDER BY
+  "_q_0"."EMPID";
+
+# title: unpivoted table source, unpivot has column aliases
+# execute: false
+SELECT * FROM (SELECT * FROM m_sales) AS m_sales(empid, dept, jan, feb) UNPIVOT(sales FOR month IN (jan, feb)) AS unpiv(a, b, c, d);
+SELECT
+  "unpiv"."a" AS "a",
+  "unpiv"."b" AS "b",
+  "unpiv"."c" AS "c",
+  "unpiv"."d" AS "d"
+FROM (
+  SELECT
+    "m_sales"."empid" AS "empid",
+    "m_sales"."dept" AS "dept",
+    "m_sales"."jan" AS "jan",
+    "m_sales"."feb" AS "feb"
+  FROM "m_sales" AS "m_sales"
+) AS "m_sales" UNPIVOT("sales" FOR "month" IN ("m_sales"."jan", "m_sales"."feb")) AS "unpiv"("a", "b", "c", "d");
+
+# title: unpivoted derived table source with a single value column
+# execute: false
+# dialect: snowflake
+SELECT * FROM (SELECT * FROM m_sales) AS m_sales(empid, dept, jan, feb) UNPIVOT(sales FOR month IN (jan, feb)) ORDER BY empid;
+SELECT
+  "_q_0"."EMPID" AS "EMPID",
+  "_q_0"."DEPT" AS "DEPT",
+  "_q_0"."MONTH" AS "MONTH",
+  "_q_0"."SALES" AS "SALES"
+FROM (
+  SELECT
+    "M_SALES"."EMPID" AS "EMPID",
+    "M_SALES"."DEPT" AS "DEPT",
+    "M_SALES"."JAN" AS "JAN",
+    "M_SALES"."FEB" AS "FEB"
+  FROM "M_SALES" AS "M_SALES"
+) AS "M_SALES" UNPIVOT("SALES" FOR "MONTH" IN ("JAN", "FEB")) AS "_q_0"
+ORDER BY
+  "_q_0"."EMPID";
+
+# title: unpivoted table source with a single value column, unpivot columns can be qualified
+# execute: false
+# dialect: bigquery
+# note: the named columns aren't supported by BQ but we add them here to avoid defining a schema
+SELECT * FROM produce AS produce(product, q1, q2, q3, q4) UNPIVOT(sales FOR quarter IN (q1, q2, q3, q4));
+SELECT
+  `_q_0`.`product` AS `product`,
+  `_q_0`.`quarter` AS `quarter`,
+  `_q_0`.`sales` AS `sales`
+FROM `produce` AS `produce` UNPIVOT(`sales` FOR `quarter` IN (`produce`.`q1`, `produce`.`q2`, `produce`.`q3`, `produce`.`q4`)) AS `_q_0`;
+
+# title: unpivoted table source with multiple value columns
+# execute: false
+# dialect: bigquery
+SELECT * FROM produce AS produce(product, q1, q2, q3, q4) UNPIVOT((first_half_sales, second_half_sales) FOR semesters IN ((Q1, Q2) AS 'semester_1', (Q3, Q4) AS 'semester_2'));
+SELECT
+  `_q_0`.`product` AS `product`,
+  `_q_0`.`semesters` AS `semesters`,
+  `_q_0`.`first_half_sales` AS `first_half_sales`,
+  `_q_0`.`second_half_sales` AS `second_half_sales`
+FROM `produce` AS `produce` UNPIVOT((`first_half_sales`, `second_half_sales`) FOR `semesters` IN ((`produce`.`q1`, `produce`.`q2`) AS 'semester_1', (`produce`.`q3`, `produce`.`q4`) AS 'semester_2')) AS `_q_0`;
+
+# title: quoting is preserved
 # dialect: snowflake
 with cte1("id", foo) as (select 1, 2) select "id" from cte1;
 WITH "CTE1" AS (
@@ -653,7 +756,7 @@ WITH "CTE1" AS (
 )
 SELECT
   "CTE1"."id" AS "id"
-FROM "CTE1";
+FROM "CTE1" AS "CTE1";
 
 # title: ensures proper quoting happens after all optimizations
 # execute: false
@@ -717,7 +820,7 @@ SELECT
   `TOp_TeRmS`.`refresh_date` AS `day`,
   `TOp_TeRmS`.`term` AS `top_term`,
   `TOp_TeRmS`.`rank` AS `rank`
-FROM `bigquery-public-data`.`GooGle_tReNDs`.`TOp_TeRmS` AS `TOp_TeRmS`
+FROM `bigquery-public-data.GooGle_tReNDs.TOp_TeRmS` AS `TOp_TeRmS`
 WHERE
   `TOp_TeRmS`.`rank` = 1
   AND CAST(`TOp_TeRmS`.`refresh_date` AS DATE) >= DATE_SUB(CURRENT_DATE, INTERVAL 2 WEEK)
@@ -987,7 +1090,7 @@ WITH "m" AS (
 )
 SELECT
   COALESCE("m"."a", "foo"."a") AS "a"
-FROM "m"
+FROM "m" AS "m"
 JOIN "n" AS "foo"("a")
   ON "foo"."a" = "m"."a";
 
@@ -1067,7 +1170,7 @@ SELECT
   "alias3"."m_od" AS "m_od",
   COALESCE("alias3"."c_od", 0) AS "c_od"
 FROM "table1" AS "table1"
-LEFT JOIN "alias3"
+LEFT JOIN "alias3" AS "alias3"
   ON "alias3"."cid" = "table1"."cid";
 
 # title: CTE with EXPLODE cannot be merged
@@ -1102,7 +1205,7 @@ WITH `T` AS (
 )
 SELECT
   `T`.`CoL` AS `CoL`
-FROM `T`;
+FROM `T` AS `T`;
 
 # title: override mysql's settings so it normalizes to lowercase
 # dialect: mysql, normalization_strategy = lowercase
@@ -1114,7 +1217,7 @@ WITH `t` AS (
 )
 SELECT
   `t`.`CoL` AS `CoL`
-FROM `t`;
+FROM `t` AS `t`;
 
 # title: top-level query is parenthesized
 # execute: false
@@ -1136,11 +1239,151 @@ WITH "x" AS (
 (
   SELECT
     "x"."a" AS "a"
-  FROM "x"
+  FROM "x" AS "x"
   UNION ALL
   SELECT
     "x"."a" AS "a"
-  FROM "x"
+  FROM "x" AS "x"
   LIMIT 10
 )
 LIMIT 10;
+
+# title: avoid producing DAG cycle when pushing down predicate to join
+# execute: false
+SELECT
+  a.company,
+  b.num
+FROM route AS a(num, company, pos, stop)
+JOIN route AS b(num, company, pos, stop) ON (a.num = b.num)
+JOIN stops AS c(id, name) ON (c.id = b.stop)
+JOIN stops AS d(id, name) ON (d.id = c.id)
+WHERE
+  c.name = 'Craiglockhart'
+  OR d.name = 'Tollcross';
+SELECT
+  "a"."company" AS "company",
+  "b"."num" AS "num"
+FROM "route" AS "a"("num", "company", "pos", "stop")
+JOIN "route" AS "b"("num", "company", "pos", "stop")
+  ON "a"."num" = "b"."num"
+JOIN "stops" AS "c"("id", "name")
+  ON "b"."stop" = "c"."id"
+JOIN "stops" AS "d"("id", "name")
+  ON "c"."id" = "d"."id"
+  AND (
+    "c"."name" = 'Craiglockhart' OR "d"."name" = 'Tollcross'
+  );
+
+# title: avoid dag cycles with unnesting subqueries
+# execute: false
+# dialect: snowflake
+SELECT
+  A.ACCOUNT_ID,
+  A.NAME,
+  C.EMAIL_DOMAIN
+FROM ACCOUNTS AS A
+LEFT JOIN CONTACTS AS C
+  ON C.ACCOUNT_ID = A.ACCOUNT_ID
+  AND C.EMAIL_DOMAIN IN (
+    SELECT
+      D.DOMAIN
+    FROM DOMAINS D
+    WHERE
+      TYPE = 'education'
+  );
+WITH "_u_0" AS (
+  SELECT
+    "D"."DOMAIN" AS "DOMAIN"
+  FROM "DOMAINS" AS "D"
+  WHERE
+    "D"."TYPE" = 'education'
+  GROUP BY
+    "D"."DOMAIN"
+)
+SELECT
+  "A"."ACCOUNT_ID" AS "ACCOUNT_ID",
+  "A"."NAME" AS "NAME",
+  "C"."EMAIL_DOMAIN" AS "EMAIL_DOMAIN"
+FROM "ACCOUNTS" AS "A"
+LEFT JOIN "CONTACTS" AS "C"
+  ON "A"."ACCOUNT_ID" = "C"."ACCOUNT_ID"
+LEFT JOIN "_u_0" AS "_u_0"
+  ON "C"."EMAIL_DOMAIN" = "_u_0"."DOMAIN"
+WHERE
+  NOT "_u_0"."DOMAIN" IS NULL;
+
+# title: decorrelate subquery and transpile ArrayAny correctly when generating spark
+# execute: false
+# dialect: spark
+SELECT
+  COUNT(DISTINCT cs1.cs_order_number) AS `order count`,
+  SUM(cs1.cs_ext_ship_cost) AS `total shipping cost`,
+  SUM(cs1.cs_net_profit) AS `total net profit`
+FROM catalog_sales cs1, date_dim, customer_address, call_center
+WHERE
+  date_dim.d_date BETWEEN '2002-02-01' AND (CAST('2002-02-01' AS DATE) + INTERVAL 60 days)
+  AND cs1.cs_ship_date_sk = date_dim.d_date_sk
+  AND cs1.cs_ship_addr_sk = customer_address.ca_address_sk
+  AND customer_address.ca_state = 'GA'
+  AND cs1.cs_call_center_sk = call_center.cc_call_center_sk
+  AND call_center.cc_county IN (
+    'Williamson County', 'Williamson County', 'Williamson County', 'Williamson County', 'Williamson County'
+  )
+  AND EXISTS(
+    SELECT *
+    FROM catalog_sales cs2
+    WHERE cs1.cs_order_number = cs2.cs_order_number
+      AND cs1.cs_warehouse_sk <> cs2.cs_warehouse_sk)
+      AND NOT EXISTS(
+        SELECT *
+        FROM catalog_returns cr1
+        WHERE cs1.cs_order_number = cr1.cr_order_number
+      )
+    ORDER BY COUNT(DISTINCT cs1.cs_order_number
+  )
+LIMIT 100;
+WITH `_u_0` AS (
+  SELECT
+    `cs2`.`cs_order_number` AS `_u_1`,
+    COLLECT_LIST(`cs2`.`cs_warehouse_sk`) AS `_u_2`
+  FROM `catalog_sales` AS `cs2`
+  GROUP BY
+    `cs2`.`cs_order_number`
+), `_u_3` AS (
+  SELECT
+    `cr1`.`cr_order_number` AS `_u_4`
+  FROM `catalog_returns` AS `cr1`
+  GROUP BY
+    `cr1`.`cr_order_number`
+)
+SELECT
+  COUNT(DISTINCT `cs1`.`cs_order_number`) AS `order count`,
+  SUM(`cs1`.`cs_ext_ship_cost`) AS `total shipping cost`,
+  SUM(`cs1`.`cs_net_profit`) AS `total net profit`
+FROM `catalog_sales` AS `cs1`
+LEFT JOIN `_u_0` AS `_u_0`
+  ON `_u_0`.`_u_1` = `cs1`.`cs_order_number`
+LEFT JOIN `_u_3` AS `_u_3`
+  ON `_u_3`.`_u_4` = `cs1`.`cs_order_number`
+JOIN `call_center` AS `call_center`
+  ON `call_center`.`cc_call_center_sk` = `cs1`.`cs_call_center_sk`
+  AND `call_center`.`cc_county` IN ('Williamson County', 'Williamson County', 'Williamson County', 'Williamson County', 'Williamson County')
+JOIN `customer_address` AS `customer_address`
+  ON `cs1`.`cs_ship_addr_sk` = `customer_address`.`ca_address_sk`
+  AND `customer_address`.`ca_state` = 'GA'
+JOIN `date_dim` AS `date_dim`
+  ON `cs1`.`cs_ship_date_sk` = `date_dim`.`d_date_sk`
+  AND `date_dim`.`d_date` <= (
+    CAST(CAST('2002-02-01' AS DATE) AS TIMESTAMP) + INTERVAL '60' DAYS
+  )
+  AND `date_dim`.`d_date` >= '2002-02-01'
+WHERE
+  `_u_3`.`_u_4` IS NULL
+  AND NOT `_u_0`.`_u_1` IS NULL
+  AND (
+    SIZE(`_u_0`.`_u_2`) = 0
+    OR SIZE(FILTER(`_u_0`.`_u_2`, `_x` -> `cs1`.`cs_warehouse_sk` <> `_x`)) <> 0
+  )
+ORDER BY
+  COUNT(DISTINCT `cs1`.`cs_order_number`)
+LIMIT 100;
