@@ -2,7 +2,7 @@ SELECT a FROM
 SELECT a FROM x WHERE
 SELECT a +
 a *
-SELECT a FROM x JOIN
+SELECT a FROM x,
 SELECT a FROM x GROUP BY
 WITH a AS (SELECT 1), b AS (SELECT 2)
 SELECT FROM x
