@@ -3,6 +3,7 @@ import unittest
 from sqlglot import (
     alias,
     and_,
+    case,
     condition,
     except_,
     exp,
@@ -77,16 +78,23 @@ class TestBuild(unittest.TestCase):
             (lambda: x.ilike("y"), "x ILIKE 'y'"),
             (lambda: x.rlike("y"), "REGEXP_LIKE(x, 'y')"),
             (
-                lambda: exp.Case().when("x = 1", "x").else_("bar"),
+                lambda: case().when("x = 1", "x").else_("bar"),
                 "CASE WHEN x = 1 THEN x ELSE bar END",
             ),
+            (
+                lambda: case("x").when("1", "x").else_("bar"),
+                "CASE x WHEN 1 THEN x ELSE bar END",
+            ),
             (lambda: exp.func("COALESCE", "x", 1), "COALESCE(x, 1)"),
+            (lambda: exp.column("x").desc(), "x DESC"),
+            (lambda: exp.column("x").desc(nulls_first=True), "x DESC NULLS FIRST"),
             (lambda: select("x"), "SELECT x"),
             (lambda: select("x"), "SELECT x"),
             (lambda: select("x", "y"), "SELECT x, y"),
             (lambda: select("x").from_("tbl"), "SELECT x FROM tbl"),
             (lambda: select("x", "y").from_("tbl"), "SELECT x, y FROM tbl"),
             (lambda: select("x").select("y").from_("tbl"), "SELECT x, y FROM tbl"),
+            (lambda: select("comment", "begin"), "SELECT comment, begin"),
             (
                 lambda: select("x").select("y", append=False).from_("tbl"),
                 "SELECT y FROM tbl",
@@ -494,6 +502,25 @@ class TestBuild(unittest.TestCase):
                 ),
                 "SELECT x FROM (SELECT x FROM tbl UNION SELECT x FROM bar) AS unioned",
             ),
+            (lambda: parse_one("(SELECT 1)").select("2"), "(SELECT 1, 2)"),
+            (
+                lambda: parse_one("(SELECT 1)").limit(1),
+                "SELECT * FROM ((SELECT 1)) AS _l_0 LIMIT 1",
+            ),
+            (
+                lambda: parse_one("WITH t AS (SELECT 1) (SELECT 1)").limit(1),
+                "SELECT * FROM (WITH t AS (SELECT 1) (SELECT 1)) AS _l_0 LIMIT 1",
+            ),
+            (
+                lambda: parse_one("(SELECT 1 LIMIT 2)").limit(1),
+                "SELECT * FROM ((SELECT 1 LIMIT 2)) AS _l_0 LIMIT 1",
+            ),
+            (lambda: parse_one("(SELECT 1)").subquery(), "((SELECT 1))"),
+            (lambda: parse_one("(SELECT 1)").subquery("alias"), "((SELECT 1)) AS alias"),
+            (
+                lambda: parse_one("(select * from foo)").with_("foo", "select 1 as c"),
+                "WITH foo AS (SELECT 1 AS c) (SELECT * FROM foo)",
+            ),
             (
                 lambda: exp.update("tbl", {"x": None, "y": {"x": 1}}),
                 "UPDATE tbl SET x = NULL, y = MAP(ARRAY('x'), ARRAY(1))",
@@ -613,6 +640,10 @@ class TestBuild(unittest.TestCase):
                 lambda: exp.insert("SELECT * FROM tbl2", "tbl"),
                 "INSERT INTO tbl SELECT * FROM tbl2",
             ),
+            (
+                lambda: exp.insert("SELECT * FROM tbl2", "tbl", returning="*"),
+                "INSERT INTO tbl SELECT * FROM tbl2 RETURNING *",
+            ),
             (
                 lambda: exp.insert("SELECT * FROM tbl2", "tbl", overwrite=True),
                 "INSERT OVERWRITE TABLE tbl SELECT * FROM tbl2",
@@ -621,6 +652,10 @@ class TestBuild(unittest.TestCase):
                 lambda: exp.insert("VALUES (1, 2), (3, 4)", "tbl", columns=["cola", "colb"]),
                 "INSERT INTO tbl (cola, colb) VALUES (1, 2), (3, 4)",
             ),
+            (
+                lambda: exp.insert("VALUES (1), (2)", "tbl", columns=["col a"]),
+                'INSERT INTO tbl ("col a") VALUES (1), (2)',
+            ),
             (
                 lambda: exp.insert("SELECT * FROM cte", "t").with_("cte", as_="SELECT x FROM tbl"),
                 "WITH cte AS (SELECT x FROM tbl) INSERT INTO t SELECT * FROM cte",
@@ -630,6 +665,26 @@ class TestBuild(unittest.TestCase):
                 "(x, y) IN ((1, 2), (3, 4))",
                 "postgres",
             ),
+            (
+                lambda: exp.cast_unless("CAST(x AS INT)", "int", "int"),
+                "CAST(x AS INT)",
+            ),
+            (
+                lambda: exp.cast_unless("CAST(x AS TEXT)", "int", "int"),
+                "CAST(CAST(x AS TEXT) AS INT)",
+            ),
+            (
+                lambda: exp.rename_column("table1", "c1", "c2", True),
+                "ALTER TABLE table1 RENAME COLUMN IF EXISTS c1 TO c2",
+            ),
+            (
+                lambda: exp.rename_column("table1", "c1", "c2", False),
+                "ALTER TABLE table1 RENAME COLUMN c1 TO c2",
+            ),
+            (
+                lambda: exp.rename_column("table1", "c1", "c2"),
+                "ALTER TABLE table1 RENAME COLUMN c1 TO c2",
+            ),
         ]:
             with self.subTest(sql):
                 self.assertEqual(expression().sql(dialect[0] if dialect else None), sql)
