@@ -243,6 +243,15 @@ class TestExpressions(unittest.TestCase):
             'SELECT * FROM a1 AS a /* a */, b.a /* b */, c.a2 /* c.a */, d2 /* d.a */ CROSS JOIN e.a CROSS JOIN "F" /* f-F.A */ CROSS JOIN g1.a /* g */',
         )
 
+        self.assertEqual(
+            exp.replace_tables(
+                parse_one("select * from example.table", dialect="bigquery"),
+                {"example.table": "`my-project.example.table`"},
+                dialect="bigquery",
+            ).sql(),
+            'SELECT * FROM "my-project"."example"."table" /* example.table */',
+        )
+
     def test_expand(self):
         self.assertEqual(
             exp.expand(
@@ -304,6 +313,18 @@ class TestExpressions(unittest.TestCase):
             ).sql(),
             "SELECT * FROM (SELECT a FROM tbl1) WHERE b > 100",
         )
+        self.assertEqual(
+            exp.replace_placeholders(
+                parse_one("select * from foo WHERE x > ? AND y IS ?"), 0, False
+            ).sql(),
+            "SELECT * FROM foo WHERE x > 0 AND y IS FALSE",
+        )
+        self.assertEqual(
+            exp.replace_placeholders(
+                parse_one("select * from foo WHERE x > :int1 AND y IS :bool1"), int1=0, bool1=False
+            ).sql(),
+            "SELECT * FROM foo WHERE x > 0 AND y IS FALSE",
+        )
 
     def test_function_building(self):
         self.assertEqual(exp.func("max", 1).sql(), "MAX(1)")
@@ -636,6 +657,7 @@ class TestExpressions(unittest.TestCase):
         self.assertIsInstance(parse_one("TO_HEX(foo)", read="bigquery"), exp.Hex)
         self.assertIsInstance(parse_one("TO_HEX(MD5(foo))", read="bigquery"), exp.MD5)
         self.assertIsInstance(parse_one("TRANSFORM(a, b)", read="spark"), exp.Transform)
+        self.assertIsInstance(parse_one("ADD_MONTHS(a, b)"), exp.AddMonths)
 
     def test_column(self):
         column = parse_one("a.b.c.d")
@@ -671,6 +693,10 @@ class TestExpressions(unittest.TestCase):
         self.assertIsInstance(parse_one("*"), exp.Star)
         self.assertEqual(exp.column("a", table="b", db="c", catalog="d"), exp.to_column("d.c.b.a"))
 
+        dot = exp.column("d", "c", "b", "a", fields=["e", "f"])
+        self.assertIsInstance(dot, exp.Dot)
+        self.assertEqual(dot.sql(), "a.b.c.d.e.f")
+
     def test_text(self):
         column = parse_one("a.b.c.d.e")
         self.assertEqual(column.text("expression"), "e")
@@ -811,14 +837,14 @@ FROM foo""",
         )
 
     def test_to_interval(self):
-        self.assertEqual(exp.to_interval("1day").sql(), "INTERVAL '1' day")
-        self.assertEqual(exp.to_interval("  5     months").sql(), "INTERVAL '5' months")
+        self.assertEqual(exp.to_interval("1day").sql(), "INTERVAL '1' DAY")
+        self.assertEqual(exp.to_interval("  5     months").sql(), "INTERVAL '5' MONTHS")
         with self.assertRaises(ValueError):
             exp.to_interval("bla")
 
-        self.assertEqual(exp.to_interval(exp.Literal.string("1day")).sql(), "INTERVAL '1' day")
+        self.assertEqual(exp.to_interval(exp.Literal.string("1day")).sql(), "INTERVAL '1' DAY")
         self.assertEqual(
-            exp.to_interval(exp.Literal.string("  5   months")).sql(), "INTERVAL '5' months"
+            exp.to_interval(exp.Literal.string("  5   months")).sql(), "INTERVAL '5' MONTHS"
         )
         with self.assertRaises(ValueError):
             exp.to_interval(exp.Literal.string("bla"))
@@ -1027,3 +1053,11 @@ FROM foo""",
         query = parse_one("SELECT * FROM foo /* sqlglot.meta x = 1, y = a, z */")
         self.assertEqual(query.find(exp.Table).meta, {"x": "1", "y": "a", "z": True})
         self.assertEqual(query.sql(), "SELECT * FROM foo /* sqlglot.meta x = 1, y = a, z */")
+
+    def test_assert_is(self):
+        parse_one("x").assert_is(exp.Column)
+
+        with self.assertRaisesRegex(
+            AssertionError, "x is not <class 'sqlglot.expressions.Identifier'>\."
+        ):
+            parse_one("x").assert_is(exp.Identifier)
