@@ -4,12 +4,13 @@ from unittest.mock import patch
 
 from sqlglot import Parser, exp, parse, parse_one
 from sqlglot.errors import ErrorLevel, ParseError
+from sqlglot.parser import logger as parser_logger
 from tests.helpers import assert_logger_contains
 
 
 class TestParser(unittest.TestCase):
     def test_parse_empty(self):
-        with self.assertRaises(ParseError) as ctx:
+        with self.assertRaises(ParseError):
             parse_one("")
 
     def test_parse_into(self):
@@ -86,6 +87,15 @@ class TestParser(unittest.TestCase):
 
         self.assertIsNotNone(parse_one("date").find(exp.Column))
 
+    def test_structs(self):
+        cast = parse_one("cast(x as struct<int>)")
+        self.assertIsInstance(cast.to.expressions[0], exp.DataType)
+        self.assertEqual(cast.sql(), "CAST(x AS STRUCT<INT>)")
+
+        cast = parse_one("cast(x as struct<varchar(10)>)")
+        self.assertIsInstance(cast.to.expressions[0], exp.DataType)
+        self.assertEqual(cast.sql(), "CAST(x AS STRUCT<VARCHAR(10)>)")
+
     def test_float(self):
         self.assertEqual(parse_one(".2"), parse_one("0.2"))
 
@@ -142,11 +152,14 @@ class TestParser(unittest.TestCase):
         )
 
     def test_command(self):
-        expressions = parse("SET x = 1; ADD JAR s3://a; SELECT 1", read="hive")
-        self.assertEqual(len(expressions), 3)
-        self.assertEqual(expressions[0].sql(), "SET x = 1")
-        self.assertEqual(expressions[1].sql(), "ADD JAR s3://a")
-        self.assertEqual(expressions[2].sql(), "SELECT 1")
+        with self.assertLogs(parser_logger) as cm:
+            expressions = parse("SET x = 1; ADD JAR s3://a; SELECT 1", read="hive")
+            self.assertEqual(len(expressions), 3)
+            self.assertEqual(expressions[0].sql(), "SET x = 1")
+            self.assertEqual(expressions[1].sql(), "ADD JAR s3://a")
+            self.assertEqual(expressions[2].sql(), "SELECT 1")
+
+        assert "'ADD JAR s3://a'" in cm.output[0]
 
     def test_lambda_struct(self):
         expression = parse_one("FILTER(a.b, x -> x.id = id)")
@@ -263,6 +276,8 @@ class TestParser(unittest.TestCase):
         with self.assertRaises(ParseError):
             parse_one("SELECT A[:")
 
+        self.assertEqual(parse_one("as as", error_level=ErrorLevel.IGNORE).sql(), "AS as")
+
     def test_space(self):
         self.assertEqual(
             parse_one("SELECT ROW() OVER(PARTITION  BY x) FROM x GROUP  BY y").sql(),
@@ -495,7 +510,9 @@ class TestParser(unittest.TestCase):
         self.assertEqual(set_to.sql(), "SET x = 1")
         self.assertIsInstance(set_to, exp.Set)
 
-        set_as_command = parse_one("SET DEFAULT ROLE ALL TO USER")
+        with self.assertLogs(parser_logger) as cm:
+            set_as_command = parse_one("SET DEFAULT ROLE ALL TO USER")
+            assert "'SET DEFAULT ROLE ALL TO USER'" in cm.output[0]
 
         self.assertEqual(set_as_command.sql(), "SET DEFAULT ROLE ALL TO USER")
 
@@ -772,7 +789,7 @@ class TestParser(unittest.TestCase):
         )
 
         self.assertEqual(ast.find(exp.Interval).this.sql(), "'71'")
-        self.assertEqual(ast.find(exp.Interval).unit.assert_is(exp.Var).sql(), "days")
+        self.assertEqual(ast.find(exp.Interval).unit.assert_is(exp.Var).sql(), "DAYS")
 
     def test_parse_concat_ws(self):
         ast = parse_one("CONCAT_WS(' ', 'John', 'Doe')")
@@ -788,3 +805,53 @@ class TestParser(unittest.TestCase):
             error_level=ErrorLevel.IGNORE,
         )
         self.assertEqual(ast[0].sql(), "CONCAT_WS()")
+
+    def test_parse_drop_schema(self):
+        for dialect in [None, "bigquery", "snowflake"]:
+            with self.subTest(dialect):
+                ast = parse_one("DROP SCHEMA catalog.schema", dialect=dialect)
+                self.assertEqual(
+                    ast,
+                    exp.Drop(
+                        this=exp.Table(
+                            this=None,
+                            db=exp.Identifier(this="schema", quoted=False),
+                            catalog=exp.Identifier(this="catalog", quoted=False),
+                        ),
+                        kind="SCHEMA",
+                    ),
+                )
+                self.assertEqual(ast.sql(dialect=dialect), "DROP SCHEMA catalog.schema")
+
+    def test_parse_create_schema(self):
+        for dialect in [None, "bigquery", "snowflake"]:
+            with self.subTest(dialect):
+                ast = parse_one("CREATE SCHEMA catalog.schema", dialect=dialect)
+                self.assertEqual(
+                    ast,
+                    exp.Create(
+                        this=exp.Table(
+                            this=None,
+                            db=exp.Identifier(this="schema", quoted=False),
+                            catalog=exp.Identifier(this="catalog", quoted=False),
+                        ),
+                        kind="SCHEMA",
+                    ),
+                )
+                self.assertEqual(ast.sql(dialect=dialect), "CREATE SCHEMA catalog.schema")
+
+    def test_values_as_identifier(self):
+        sql = "SELECT values FROM t WHERE values + 1 > x"
+        for dialect in (
+            "bigquery",
+            "clickhouse",
+            "duckdb",
+            "postgres",
+            "redshift",
+            "snowflake",
+        ):
+            with self.subTest(dialect):
+                self.assertEqual(parse_one(sql, dialect=dialect).sql(dialect=dialect), sql)
+
+    def test_distinct_from(self):
+        self.assertIsInstance(parse_one("a IS DISTINCT FROM b OR c IS DISTINCT FROM d"), exp.Or)
