@@ -6,8 +6,7 @@ from sqlglot.optimizer.annotate_types import annotate_types
 from tests.helpers import load_sql_fixtures
 
 
-class CustomExpression(exp.Expression):
-    ...
+class CustomExpression(exp.Expression): ...
 
 
 class TestSerDe(unittest.TestCase):
@@ -27,7 +26,13 @@ class TestSerDe(unittest.TestCase):
         self.assertEqual(before, after)
 
     def test_type_annotations(self):
-        before = annotate_types(parse_one("CAST('1' AS INT)"))
+        before = annotate_types(parse_one("CAST('1' AS STRUCT<x ARRAY<INT>>)"))
         after = self.dump_load(before)
         self.assertEqual(before.type, after.type)
         self.assertEqual(before.this.type, after.this.type)
+
+    def test_meta(self):
+        before = parse_one("SELECT * FROM X")
+        before.meta["x"] = 1
+        after = self.dump_load(before)
+        self.assertEqual(before.meta, after.meta)
