@@ -5,6 +5,7 @@ from unittest import mock
 from sqlglot import parse_one, transpile
 from sqlglot.errors import ErrorLevel, ParseError, UnsupportedError
 from sqlglot.helper import logger as helper_logger
+from sqlglot.parser import logger as parser_logger
 from tests.helpers import (
     assert_logger_contains,
     load_sql_fixture_pairs,
@@ -92,6 +93,10 @@ class TestTranspile(unittest.TestCase):
         self.validate("SELECT a\r\nFROM b", "SELECT a FROM b")
 
     def test_comments(self):
+        self.validate(
+            "SELECT c /* foo */ AS alias",
+            "SELECT c AS alias /* foo */",
+        )
         self.validate(
             "SELECT c AS /* foo */ (a, b, c) FROM t",
             "SELECT c AS (a, b, c) /* foo */ FROM t",
@@ -366,7 +371,7 @@ WHERE
             """,
             """SELECT
   col, /* This is testing comments */
-  CASE WHEN a THEN b ELSE c END /* 2nd testing comments */ AS d
+  CASE WHEN a THEN b ELSE c END AS d /* 2nd testing comments */
 FROM t""",
             pretty=True,
         )
@@ -549,11 +554,12 @@ FROM base""",
         self.validate(
             "WITH A(filter) AS (VALUES 1, 2, 3) SELECT * FROM A WHERE filter >= 2",
             "WITH A(filter) AS (VALUES (1), (2), (3)) SELECT * FROM A WHERE filter >= 2",
+            read="presto",
         )
         self.validate(
             "SELECT BOOL_OR(a > 10) FROM (VALUES 1, 2, 15) AS T(a)",
             "SELECT BOOL_OR(a > 10) FROM (VALUES (1), (2), (15)) AS T(a)",
-            write="presto",
+            read="presto",
         )
 
     def test_alter(self):
@@ -571,11 +577,11 @@ FROM base""",
         )
 
     def test_time(self):
-        self.validate("INTERVAL '1 day'", "INTERVAL '1' day")
-        self.validate("INTERVAL '1 days' * 5", "INTERVAL '1' days * 5")
-        self.validate("5 * INTERVAL '1 day'", "5 * INTERVAL '1' day")
-        self.validate("INTERVAL 1 day", "INTERVAL '1' day")
-        self.validate("INTERVAL 2 months", "INTERVAL '2' months")
+        self.validate("INTERVAL '1 day'", "INTERVAL '1' DAY")
+        self.validate("INTERVAL '1 days' * 5", "INTERVAL '1' DAYS * 5")
+        self.validate("5 * INTERVAL '1 day'", "5 * INTERVAL '1' DAY")
+        self.validate("INTERVAL 1 day", "INTERVAL '1' DAY")
+        self.validate("INTERVAL 2 months", "INTERVAL '2' MONTHS")
         self.validate("TIMESTAMP '2020-01-01'", "CAST('2020-01-01' AS TIMESTAMP)")
         self.validate("TIMESTAMP WITH TIME ZONE '2020-01-01'", "CAST('2020-01-01' AS TIMESTAMPTZ)")
         self.validate(
@@ -732,6 +738,43 @@ FROM base""",
             with self.subTest(sql):
                 self.assertEqual(transpile(sql)[0], sql.strip())
 
+    def test_command_identity(self):
+        for sql in (
+            "ALTER AGGREGATE bla(foo) OWNER TO CURRENT_USER",
+            "ALTER DOMAIN foo VALIDATE CONSTRAINT bla",
+            "ALTER ROLE CURRENT_USER WITH REPLICATION",
+            "ALTER RULE foo ON bla RENAME TO baz",
+            "ALTER SEQUENCE IF EXISTS baz RESTART WITH boo",
+            "ALTER SESSION SET STATEMENT_TIMEOUT_IN_SECONDS=3",
+            "ALTER TABLE integers DROP PRIMARY KEY",
+            "ALTER TABLE table1 MODIFY COLUMN name1 SET TAG foo='bar'",
+            "ALTER TABLE table1 RENAME COLUMN c1 AS c2",
+            "ALTER TABLE table1 RENAME COLUMN c1 TO c2, c2 TO c3",
+            "ALTER TABLE table1 RENAME COLUMN c1 c2",
+            "ALTER TYPE electronic_mail RENAME TO email",
+            "ALTER VIEW foo ALTER COLUMN bla SET DEFAULT 'NOT SET'",
+            "ALTER schema doo",
+            "ANALYZE a.y",
+            "CALL catalog.system.iceberg_procedure_name(named_arg_1 => 'arg_1', named_arg_2 => 'arg_2')",
+            "COMMENT ON ACCESS METHOD gin IS 'GIN index access method'",
+            "CREATE OR REPLACE STAGE",
+            "CREATE SET GLOBAL TEMPORARY TABLE a, NO BEFORE JOURNAL, NO AFTER JOURNAL, MINIMUM DATABLOCKSIZE, BLOCKCOMPRESSION=NEVER (a INT)",
+            "EXECUTE statement",
+            "EXPLAIN SELECT * FROM x",
+            "GRANT INSERT ON foo TO bla",
+            "LOAD foo",
+            "OPTIMIZE TABLE y",
+            "PREPARE statement",
+            "SET -v",
+            "SET @user OFF",
+            "SHOW TABLES",
+            "VACUUM FREEZE my_table",
+        ):
+            with self.subTest(sql):
+                with self.assertLogs(parser_logger) as cm:
+                    self.assertEqual(transpile(sql)[0], sql)
+                    assert f"'{sql[:100]}' contains unsupported syntax" in cm.output[0]
+
     def test_normalize_name(self):
         self.assertEqual(
             transpile("cardinality(x)", read="presto", write="presto", normalize_functions="lower")[
